import os
import time
import asyncio
import logging
import json
from typing import Dict, List, Optional, Set, Callable, Any

from .heartbeat import HeartbeatManager

logger = logging.getLogger(__name__)


# ============================================================================
# Worker 状态管理
# ============================================================================

class WorkerManager:
    """Worker管理器 - Worker状态的唯一管理入口

    ⚠️ 重要：所有Worker状态的修改都必须通过这个类进行，不要直接操作Redis！

    职责:
    1. Worker ID 生成和复用
    2. Worker 注册表管理
    3. Worker 状态字段的读写
    4. Redis Pub/Sub 状态变更通知
    5. Worker 查询和统计
    """

    def __init__(self, redis_client, async_redis_client, redis_prefix: str = "jettask", event_pool=None):
        """初始化Worker状态管理器

        Args:
            redis_client: 同步Redis客户端
            async_redis_client: 异步Redis客户端
            redis_prefix: Redis key前缀
            event_pool: EventPool实例（可选），用于事件驱动的消息恢复
        """
        self.sync_redis = redis_client
        self.redis = async_redis_client
        self.async_redis = async_redis_client
        self.redis_prefix = redis_prefix
        self.active_workers_key = f"{redis_prefix}:ACTIVE_WORKERS"
        self.workers_registry_key = f"{redis_prefix}:REGISTRY:WORKERS"
        self.event_pool = event_pool
        self.worker_prefix = 'WORKER'

        # Pub/Sub通道名称
        self.worker_state_channel = f"{redis_prefix}:WORKER_STATE_CHANGE"

        # 监听器订阅
        self._pubsub = None
        self._listener_task: Optional[asyncio.Task] = None
        self._running = False
        self._callbacks: Set[Callable] = set()

        # Pub/Sub 配置
        self._health_check_interval = 60
        self._health_check_task: Optional[asyncio.Task] = None

        # Worker扫描器相关（从 WorkerScanner 合并）
        self._initialized = False
        self._last_full_sync = 0
        self._full_sync_interval = 60
        self._scan_counter = 0
        self._partial_check_interval = 10

    def _get_worker_key(self, worker_id: str) -> str:
        """获取worker的Redis key"""
        return f"{self.redis_prefix}:WORKER:{worker_id}"

    async def initialize_worker(self, worker_id: str, worker_info: Dict[str, Any]):
        """初始化worker（首次创建）"""
        worker_key = self._get_worker_key(worker_id)
        current_time = time.time()

        worker_info.setdefault('is_alive', 'true')
        worker_info.setdefault('messages_transferred', 'false')
        worker_info.setdefault('created_at', str(current_time))
        worker_info.setdefault('last_heartbeat', str(current_time))

        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, mapping=worker_info)
        pipeline.zadd(self.active_workers_key, {worker_id: current_time})
        await pipeline.execute()

        logger.debug(f"Initialized worker {worker_id}")

    async def set_worker_online(self, worker_id: str, worker_data: dict = None):
        """设置worker为在线状态"""
        worker_key = self._get_worker_key(worker_id)
        old_alive = await self.redis.hget(worker_key, 'is_alive')
        old_alive = old_alive.decode('utf-8') if isinstance(old_alive, bytes) else old_alive

        current_time = time.time()
        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, 'is_alive', 'true')
        pipeline.hset(worker_key, 'last_heartbeat', str(current_time))

        # 当 worker 从离线变为在线时，重置 messages_transferred
        # 这表示是一个新的 worker 实例，还没有进行消息转移
        if old_alive != 'true':
            pipeline.hset(worker_key, 'messages_transferred', 'false')

        if worker_data:
            pipeline.hset(worker_key, mapping=worker_data)

        pipeline.zadd(self.active_workers_key, {worker_id: current_time})
        await pipeline.execute()

        if old_alive != 'true':
            await self._publish_state_change(worker_id, 'online')
            logger.debug(f"Worker {worker_id} is now ONLINE")

    async def set_worker_offline(self, worker_id: str, reason: str = "unknown"):
        """设置worker为离线状态"""
        worker_key = self._get_worker_key(worker_id)
        old_alive = await self.redis.hget(worker_key, 'is_alive')
        old_alive = old_alive.decode('utf-8') if isinstance(old_alive, bytes) else old_alive

        current_time = time.time()
        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, 'messages_transferred', 'false')  # 重置消息转移标记，允许其他worker接管消息
        pipeline.hset(worker_key, 'is_alive', 'false')
        pipeline.hset(worker_key, 'offline_reason', reason)
        pipeline.hset(worker_key, 'offline_time', str(current_time))
        pipeline.zrem(self.active_workers_key, worker_id)
        await pipeline.execute()

        if old_alive == 'true':
            await self._publish_state_change(worker_id, 'offline', reason)
            logger.debug(f"Worker {worker_id} is now OFFLINE (reason: {reason})")

    async def update_worker_heartbeat(self, worker_id: str, heartbeat_data: dict = None):
        """更新worker心跳（确保在线状态）"""
        worker_key = self._get_worker_key(worker_id)
        current_time = time.time()

        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, 'is_alive', 'true')
        pipeline.hset(worker_key, 'last_heartbeat', str(current_time))

        if heartbeat_data:
            pipeline.hset(worker_key, mapping=heartbeat_data)

        pipeline.zadd(self.active_workers_key, {worker_id: current_time})
        await pipeline.execute()

    async def update_worker_field(self, worker_id: str, field: str, value: str):
        """更新worker的单个字段"""
        worker_key = self._get_worker_key(worker_id)
        await self.redis.hset(worker_key, field, value)

    async def update_worker_fields(self, worker_id: str, fields: Dict[str, Any]):
        """批量更新worker的多个字段"""
        worker_key = self._get_worker_key(worker_id)
        await self.redis.hset(worker_key, mapping=fields)

    async def record_group_info(
        self,
        worker_id: str,
        queue: str,
        task_name: str,
        group_name: str,
        consumer_name: str,
        redis_prefix: str
    ):
        """记录 task 的 group 信息到 worker hash 表

        Args:
            worker_id: Worker ID
            queue: 队列名（不带前缀）
            task_name: 任务名
            group_name: Consumer group 名称
            consumer_name: Consumer 名称
            redis_prefix: Redis 前缀
        """
        try:
            worker_key = self._get_worker_key(worker_id)

            # 构建 group 信息
            group_info = {
                'queue': queue,
                'task_name': task_name,
                'group_name': group_name,
                'consumer_name': consumer_name,
                'stream_key': f"{redis_prefix}:QUEUE:{queue}"
            }

            # 将 group 信息存储到 worker 的 hash 中
            field_name = f"group_info:{group_name}"
            await self.redis.hset(
                worker_key,
                field_name,
                json.dumps(group_info)
            )
            logger.debug(f"Recorded group info for worker {worker_id}, task {task_name}: {group_info}")

        except Exception as e:
            logger.error(f"Error recording group info for worker {worker_id}: {e}", exc_info=True)

    def update_worker_field_sync(self, worker_id: str, field: str, value: str):
        """更新worker的单个字段（同步版本，供心跳线程使用）"""
        worker_key = self._get_worker_key(worker_id)
        self.sync_redis.hset(worker_key, field, value)
        logger.debug(f"Updated worker {worker_id} field {field}={value}")

    def update_worker_fields_sync(self, worker_id: str, fields: Dict[str, Any]):
        """批量更新worker的多个字段（同步版本，供心跳线程使用）"""
        worker_key = self._get_worker_key(worker_id)
        self.sync_redis.hset(worker_key, mapping=fields)
        logger.debug(f"Updated worker {worker_id} fields: {list(fields.keys())}")

    async def increment_queue_stats(self, worker_id: str, queue: str,
                                   running_tasks_delta: int = None,
                                   success_count_increment: int = None,
                                   failed_count_increment: int = None,
                                   total_count_increment: int = None,
                                   processing_time_increment: float = None,
                                   latency_time_increment: float = None):
        """增量更新worker在特定队列上的累积统计信息"""
        worker_key = self._get_worker_key(worker_id)
        pipeline = self.redis.pipeline()

        if running_tasks_delta is not None and running_tasks_delta != 0:
            pipeline.hincrby(worker_key, f'{queue}:running_tasks', running_tasks_delta)

        if success_count_increment is not None:
            pipeline.hincrby(worker_key, f'{queue}:success_count', success_count_increment)

        if failed_count_increment is not None:
            pipeline.hincrby(worker_key, f'{queue}:failed_count', failed_count_increment)

        if total_count_increment is not None:
            pipeline.hincrby(worker_key, f'{queue}:total_count', total_count_increment)

        if processing_time_increment is not None:
            pipeline.hincrbyfloat(worker_key, f'{queue}:total_processing_time', processing_time_increment)

        if latency_time_increment is not None:
            pipeline.hincrbyfloat(worker_key, f'{queue}:total_latency_time', latency_time_increment)

        await pipeline.execute()

    async def get_queue_total_stats(self, worker_id: str, queue: str) -> dict:
        """获取队列的累积统计数据"""
        worker_key = self._get_worker_key(worker_id)
        fields = [
            f'{queue}:total_count',
            f'{queue}:total_processing_time',
            f'{queue}:total_latency_time'
        ]
        values = await self.redis.hmget(worker_key, fields)

        return {
            'total_count': int(values[0]) if values[0] else 0,
            'total_processing_time': float(values[1]) if values[1] else 0.0,
            'total_latency_time': float(values[2]) if values[2] else 0.0
        }

    async def update_queue_stats(self, worker_id: str, queue: str,
                                 running_tasks: int = None,
                                 avg_processing_time: float = None,
                                 avg_latency_time: float = None):
        """更新worker在特定队列上的统计信息"""
        worker_key = self._get_worker_key(worker_id)
        pipeline = self.redis.pipeline()

        if running_tasks is not None:
            pipeline.hset(worker_key, f'{queue}:running_tasks', str(running_tasks))

        if avg_processing_time is not None:
            pipeline.hset(worker_key, f'{queue}:avg_processing_time', f'{avg_processing_time:.3f}')

        if avg_latency_time is not None:
            pipeline.hset(worker_key, f'{queue}:avg_latency_time', f'{avg_latency_time:.3f}')

        await pipeline.execute()

    async def mark_messages_transferred(self, worker_id: str, transferred: bool = True):
        """标记worker的消息是否已转移"""
        worker_key = self._get_worker_key(worker_id)
        await self.redis.hset(worker_key, 'messages_transferred', 'true' if transferred else 'false')

    async def get_worker_info(self, worker_id: str) -> Optional[Dict[str, str]]:
        """获取worker的完整信息"""
        worker_key = self._get_worker_key(worker_id)
        data = await self.redis.hgetall(worker_key)

        if not data:
            return None

        result = {}
        for k, v in data.items():
            key = k.decode('utf-8') if isinstance(k, bytes) else k
            value = v.decode('utf-8') if isinstance(v, bytes) else v
            result[key] = value

        return result

    async def get_worker_field(self, worker_id: str, field: str) -> Optional[str]:
        """获取worker的单个字段值"""
        worker_key = self._get_worker_key(worker_id)
        value = await self.redis.hget(worker_key, field)

        if value is None:
            return None

        return value.decode('utf-8') if isinstance(value, bytes) else value

    async def is_worker_alive(self, worker_id: str) -> bool:
        """检查worker是否在线"""
        is_alive = await self.get_worker_field(worker_id, 'is_alive')
        return is_alive == 'true'

    # ========== Worker ID 生成和复用 ==========

    def generate_worker_id(self, prefix: str) -> str:
        """
        生成新的 Worker ID

        格式: {prefix}-{uuid}-{pid}
        例如: YYDG-a1b2c3d4-12345

        Args:
            prefix: Worker ID 前缀（通常是主机名）

        Returns:
            生成的 Worker ID
        """
        import uuid
        return f"{prefix}-{uuid.uuid4().hex[:8]}-{os.getpid()}"

    async def find_reusable_worker_id(self, prefix: str) -> Optional[str]:
        """
        查找可复用的离线 Worker ID

        Args:
            prefix: Worker ID 前缀

        Returns:
            可复用的 Worker ID，如果没有则返回 None
        """
        try:
            offline_workers = await self.get_offline_workers()

            for worker_id in offline_workers:
                if isinstance(worker_id, bytes):
                    worker_id = worker_id.decode('utf-8')
                if worker_id.startswith(prefix):
                    logger.debug(f"Found reusable worker ID: {worker_id}")
                    return worker_id
        except Exception as e:
            logger.warning(f"Error finding reusable worker ID: {e}")

        return None

    # ========== Worker 注册表管理 ==========

    async def register_worker(self, worker_id: str):
        """注册 Worker 到全局注册表"""
        await self.async_redis.sadd(self.workers_registry_key, worker_id)
        logger.debug(f"Registered worker: {worker_id}")

    async def unregister_worker(self, worker_id: str):
        """从全局注册表注销 Worker"""
        await self.async_redis.srem(self.workers_registry_key, worker_id)
        logger.debug(f"Unregistered worker: {worker_id}")

    async def get_all_workers(self) -> Set[str]:
        """获取所有已注册的 Worker ID"""
        return await self.async_redis.smembers(self.workers_registry_key)

    def get_all_workers_sync(self) -> Set[str]:
        """同步方式获取所有已注册的 Worker ID"""
        return self.sync_redis.smembers(self.workers_registry_key)

    async def get_worker_count(self) -> int:
        """获取已注册的 Worker 总数"""
        return await self.async_redis.scard(self.workers_registry_key)

    async def get_offline_workers(self) -> Set[str]:
        """获取所有离线的 Worker ID

        离线 Worker 是指已注册但 is_alive=false 的 Worker
        """
        all_workers = await self.get_all_workers()
        offline_workers = set()

        for worker_id in all_workers:
            if isinstance(worker_id, bytes):
                worker_id = worker_id.decode('utf-8')

            worker_key = f"{self.redis_prefix}:WORKER:{worker_id}"
            is_alive = await self.async_redis.hget(worker_key, 'is_alive')

            if is_alive:
                is_alive = is_alive.decode('utf-8') if isinstance(is_alive, bytes) else is_alive
                if is_alive != 'true':
                    offline_workers.add(worker_id)
            else:
                # Worker key 不存在或没有 is_alive 字段，认为离线
                offline_workers.add(worker_id)

        return offline_workers

    async def get_workers_for_task(self, task_name: str, only_alive: bool = True) -> Set[str]:
        """获取执行特定任务的 Worker 列表

        通过检查 WORKER:* hash 中的 group_info 字段来判断哪些 Worker 在处理该任务

        Args:
            task_name: 任务名称
            only_alive: 是否只返回在线的 Worker（默认 True）

        Returns:
            处理该任务的 Worker ID 集合
        """
        all_worker_ids = await self.get_all_workers()
        matched_workers = set()
        group_info_prefix = f"group_info:{self.redis_prefix}:QUEUE:"

        for worker_id in all_worker_ids:
            if isinstance(worker_id, bytes):
                worker_id = worker_id.decode('utf-8')

            worker_key = f"{self.redis_prefix}:WORKER:{worker_id}"
            worker_info = await self.async_redis.hgetall(worker_key)

            if not worker_info:
                continue

            # 解码 bytes keys
            decoded_info = {}
            for k, v in worker_info.items():
                key = k.decode('utf-8') if isinstance(k, bytes) else k
                val = v.decode('utf-8') if isinstance(v, bytes) else v
                decoded_info[key] = val

            # 检查 is_alive 状态
            if only_alive:
                is_alive = decoded_info.get('is_alive', 'false')
                if is_alive != 'true':
                    continue

            # 检查是否包含该任务的 group_info
            # 格式: group_info:test5:QUEUE:robust_bench2:benchmark_task
            for key in decoded_info.keys():
                if key.startswith(group_info_prefix):
                    parts = key.split(':')
                    if len(parts) >= 5:
                        worker_task_name = parts[-1]  # 最后一部分是 task_name
                        if worker_task_name == task_name:
                            matched_workers.add(worker_id)
                            break

        return matched_workers

    async def get_active_worker_count_for_task(self, task_name: str) -> int:
        """获取执行特定任务的在线 Worker 数量

        Args:
            task_name: 任务名称

        Returns:
            在线 Worker 数量
        """
        workers = await self.get_workers_for_task(task_name, only_alive=True)
        return len(workers)

    async def find_offline_workers_for_task(
        self,
        task_name: str,
        worker_prefix: str = 'WORKER'
    ) -> List[tuple]:
        """查找指定任务的离线 Worker

        查找条件：
        1. Worker 已离线（is_alive=false）
        2. 消息未转移（messages_transferred=false）
        3. Worker 的 group_info 中有该任务

        Args:
            task_name: 任务名称
            worker_prefix: Worker 键前缀（默认 'WORKER'）

        Returns:
            离线 Worker 列表，每项为 (worker_key, worker_data) 元组
        """
        offline_workers = []

        try:
            # 获取所有 worker ID
            worker_ids = await self.get_all_workers()
            logger.debug(f"[Recovery] Found {len(worker_ids)} workers in registry for task {task_name}")

            for worker_id in worker_ids:
                # 解码 worker_id（可能是 bytes）
                if isinstance(worker_id, bytes):
                    worker_id = worker_id.decode('utf-8')

                # 构建 worker key
                worker_key = f"{self.redis_prefix}:{worker_prefix}:{worker_id}"

                try:
                    # 直接使用自身的方法读取 worker 信息
                    decoded_worker_data = await self.get_worker_info(worker_id)

                    if not decoded_worker_data:
                        continue

                    # 检查 worker 是否离线且消息未转移
                    is_alive = decoded_worker_data.get('is_alive', 'false') == 'true'
                    messages_transferred = decoded_worker_data.get('messages_transferred', 'false') == 'true'

                    # 找到离线且消息未转移的 worker
                    if not is_alive and not messages_transferred:
                        # 检查 worker 的 group_info 中是否有该任务
                        has_task = False
                        for key, value in decoded_worker_data.items():
                            if key.startswith('group_info:'):
                                try:
                                    import json
                                    group_info = json.loads(value)
                                    if group_info.get('task_name') == task_name:
                                        has_task = True
                                        break
                                except Exception as e:
                                    logger.error(f"Error parsing group_info: {e}")

                        if has_task:
                            logger.info(
                                f"Found offline worker needing recovery: {worker_id}, "
                                f"task={task_name}, is_alive={is_alive}, "
                                f"messages_transferred={messages_transferred}"
                            )
                            offline_workers.append((worker_key, decoded_worker_data))
                        else:
                            logger.debug(
                                f"Worker {worker_id} is offline but not responsible for task {task_name}"
                            )

                except Exception as e:
                    logger.error(f"Error processing worker key {worker_key}: {e}")
                    continue

        except Exception as e:
            logger.error(f"Error finding offline workers: {e}")

        return offline_workers

    async def get_all_workers_info(self, only_alive: bool = True) -> Dict[str, Dict[str, str]]:
        """获取所有worker的信息"""
        pattern = f"{self.redis_prefix}:WORKER:*"
        result = {}

        cursor = 0
        while True:
            cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)

            for key in keys:
                if isinstance(key, bytes):
                    key = key.decode('utf-8')

                parts = key.split(":")
                if len(parts) >= 3:
                    worker_id = parts[2]
                    worker_info = await self.get_worker_info(worker_id)
                    if worker_info:
                        if only_alive and worker_info.get('is_alive') != 'true':
                            continue
                        result[worker_id] = worker_info

            if cursor == 0:
                break

        return result

    async def delete_worker(self, worker_id: str):
        """删除worker的所有数据"""
        worker_key = self._get_worker_key(worker_id)
        pipeline = self.redis.pipeline()
        pipeline.delete(worker_key)
        pipeline.zrem(self.active_workers_key, worker_id)
        await pipeline.execute()
        logger.debug(f"Deleted worker {worker_id}")

    async def _publish_state_change(self, worker_id: str, state: str, reason: str = None):
        """发布状态变更信号"""
        message = {
            'worker_id': worker_id,
            'state': state,
            'timestamp': asyncio.get_event_loop().time()
        }

        if reason:
            message['reason'] = reason

        await self.redis.publish(
            self.worker_state_channel,
            json.dumps(message)
        )

        logger.debug(f"Published state change: {message}")

    async def start_listener(self):
        """启动状态变更监听器"""
        if self._running:
            logger.warning("Worker state listener already running")
            return

        self._running = True
        self._pubsub = await self._create_and_subscribe_pubsub()
        self._listener_task = asyncio.create_task(self._listen_loop())
        self._health_check_task = asyncio.create_task(self._health_check_loop())

        logger.debug(f"Started worker state listener on channel: {self.worker_state_channel}")

    async def stop_listener(self):
        """停止状态变更监听器"""
        if not self._running:
            return

        self._running = False

        if self._listener_task:
            self._listener_task.cancel()
            try:
                await self._listener_task
            except asyncio.CancelledError:
                pass

        if self._health_check_task:
            self._health_check_task.cancel()
            try:
                await self._health_check_task
            except asyncio.CancelledError:
                pass

        if self._pubsub:
            await self._pubsub.unsubscribe(self.worker_state_channel)
            await self._pubsub.close()

        logger.debug("Stopped worker state listener")

    async def _create_and_subscribe_pubsub(self):
        """创建 PubSub 连接并订阅频道"""
        if self._pubsub:
            try:
                await self._pubsub.close()
            except:
                pass

        pubsub = self.redis.pubsub()
        await pubsub.subscribe(self.worker_state_channel)

        # 标记PubSub连接，防止被空闲连接清理
        if hasattr(pubsub, 'connection') and pubsub.connection:
            pubsub.connection._is_pubsub_connection = True
            socket_timeout = pubsub.connection.socket_timeout if hasattr(pubsub.connection, 'socket_timeout') else 'N/A'
            logger.info(f"Marked PubSub connection {id(pubsub.connection)} to prevent cleanup, socket_timeout={socket_timeout}")

        logger.debug(f"Created and subscribed to Redis Pub/Sub channel: {self.worker_state_channel}")
        return pubsub

    async def _health_check_loop(self):
        """定期检查 Pub/Sub 连接健康状态"""
        while self._running:
            try:
                await asyncio.sleep(self._health_check_interval)

                if not self._running:
                    break

                if self._pubsub and self._pubsub.connection:
                    try:
                        await asyncio.wait_for(self._pubsub.ping(), timeout=5.0)
                        logger.debug("Pub/Sub health check: OK")
                    except Exception as e:
                        logger.warning(f"Pub/Sub health check failed: {e}")
                else:
                    logger.warning("Pub/Sub connection is None")

            except asyncio.CancelledError:
                logger.debug("Health check loop cancelled")
                break
            except Exception as e:
                logger.error(f"Error in health check loop: {e}")

    async def _listen_loop(self):
        """监听循环（支持自动重连）"""
        retry_delay = 1
        max_retry_delay = 30

        while self._running:
            try:
                async for message in self._pubsub.listen():
                    if message['type'] == 'message':
                        try:
                            data = json.loads(message['data'])

                            if data.get('state') == 'offline' and self.event_pool:
                                worker_id = data.get('worker_id')
                                if worker_id:
                                    logger.info(f"[StateManager] Worker {worker_id} offline event received")
                                    asyncio.create_task(
                                        self.event_pool.handle_worker_offline_event(worker_id)
                                    )

                            for callback in self._callbacks:
                                try:
                                    if asyncio.iscoroutinefunction(callback):
                                        await callback(data)
                                    else:
                                        callback(data)
                                except Exception as e:
                                    logger.error(f"Error in state change callback: {e}")

                        except Exception as e:
                            logger.error(f"Error processing state change message: {e}")

                retry_delay = 1

            except asyncio.CancelledError:
                logger.debug("Listen loop cancelled")
                break
            except Exception as e:
                logger.error(f"Error in listen loop: {e}")

                if not self._running:
                    break

                logger.warning(f"Attempting to reconnect to Redis Pub/Sub in {retry_delay} seconds...")
                await asyncio.sleep(retry_delay)

                try:
                    self._pubsub = await self._create_and_subscribe_pubsub()
                    logger.info(f"Successfully reconnected to Redis Pub/Sub")
                    retry_delay = 1
                except Exception as reconnect_error:
                    logger.error(f"Failed to reconnect to Redis Pub/Sub: {reconnect_error}")
                    retry_delay = min(retry_delay * 2, max_retry_delay)

        logger.debug("Listen loop exited")

    def register_callback(self, callback: Callable):
        """注册状态变更回调"""
        self._callbacks.add(callback)
        logger.debug(f"Registered state change callback: {callback.__name__}")

    def unregister_callback(self, callback: Callable):
        """注销状态变更回调"""
        self._callbacks.discard(callback)
        logger.debug(f"Unregistered state change callback: {callback.__name__}")

    # ========================================================================
    # Worker 扫描和超时检测（从 WorkerScanner 合并）
    # ========================================================================

    async def scan_timeout_workers(self) -> List[Dict]:
        """快速扫描超时的 worker - O(log N) 复杂度

        每个 worker 使用自己存储的 heartbeat_timeout 值进行超时判断。
        这个值由 HeartbeatManager 在 worker 初始化时设置。

        Returns:
            超时的 worker 列表
        """
        self._scan_counter += 1
        if self._scan_counter >= self._partial_check_interval:
            self._scan_counter = 0
            asyncio.create_task(self._partial_check())

        current_time = time.time()

        potential_timeout_worker_ids = await self.async_redis.zrangebyscore(
            self.active_workers_key,
            min=0,
            max=current_time - 1
        )

        if not potential_timeout_worker_ids:
            return []

        # 直接使用自身的方法获取 worker 信息
        all_workers_info = await self.get_all_workers_info(only_alive=False)
        workers_data = [all_workers_info.get(wid) for wid in potential_timeout_worker_ids]

        result = []
        cleanup_pipeline = self.async_redis.pipeline()
        need_cleanup = False

        for worker_id, worker_data in zip(potential_timeout_worker_ids, workers_data):
            if not worker_data:
                cleanup_pipeline.zrem(self.active_workers_key, worker_id)
                cleanup_pipeline.srem(self.workers_registry_key, worker_id)
                need_cleanup = True
                continue

            # 使用 worker 自己存储的 heartbeat_timeout
            # 这个值在 HeartbeatManager 初始化 worker 时设置
            worker_heartbeat_timeout = float(worker_data.get('heartbeat_timeout', 15.0))
            last_heartbeat = float(worker_data.get('last_heartbeat', 0))
            worker_cutoff_time = current_time - worker_heartbeat_timeout

            if last_heartbeat >= worker_cutoff_time:
                cleanup_pipeline.zadd(self.active_workers_key, {worker_id: last_heartbeat})
                need_cleanup = True
                continue

            is_alive = worker_data.get('is_alive', 'true') == 'true'
            if not is_alive:
                cleanup_pipeline.zrem(self.active_workers_key, worker_id)
                need_cleanup = True
                continue

            logger.debug(f"Worker {worker_id} timeout: last_heartbeat={last_heartbeat}, timeout={worker_heartbeat_timeout}s")
            worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"
            result.append({
                'worker_key': worker_key,
                'worker_data': worker_data,
                'worker_id': worker_id
            })

        if need_cleanup:
            await cleanup_pipeline.execute()

        if result:
            logger.info(f"Found {len(result)} timeout workers")

        return result

    async def update_heartbeat(self, worker_id: str, heartbeat_time: Optional[float] = None):
        """原子性更新心跳"""
        if heartbeat_time is None:
            heartbeat_time = time.time()

        pipeline = self.async_redis.pipeline()
        worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"

        pipeline.hset(worker_key, 'last_heartbeat', str(heartbeat_time))
        pipeline.zadd(self.active_workers_key, {worker_id: heartbeat_time})

        await pipeline.execute()

    async def add_worker(self, worker_id: str, worker_data: Dict):
        """添加新 worker"""
        heartbeat_time = float(worker_data.get('last_heartbeat', time.time()))

        pipeline = self.async_redis.pipeline()
        worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"

        pipeline.hset(worker_key, mapping=worker_data)
        pipeline.zadd(self.active_workers_key, {worker_id: heartbeat_time})

        await pipeline.execute()
        logger.debug(f"Added worker {worker_id} to system")

    async def remove_worker(self, worker_id: str):
        """移除 worker（标记为离线并从活跃集合中移除）"""
        await self.set_worker_offline(worker_id, reason="heartbeat_timeout")
        await self.async_redis.zrem(self.active_workers_key, worker_id)

    async def cleanup_stale_workers(self, max_age_seconds: float = 3600):
        """清理过期的 worker 记录"""
        current_time = time.time()
        cutoff_time = current_time - max_age_seconds

        stale_worker_ids = await self.async_redis.zrangebyscore(
            self.active_workers_key,
            min=0,
            max=cutoff_time
        )

        if not stale_worker_ids:
            return 0

        pipeline = self.async_redis.pipeline()

        for worker_id in stale_worker_ids:
            worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"
            pipeline.delete(worker_key)

        pipeline.zrem(self.active_workers_key, *stale_worker_ids)

        await pipeline.execute()

        logger.info(f"Cleaned up {len(stale_worker_ids)} stale worker records")
        return len(stale_worker_ids)

    async def _partial_check(self):
        """部分一致性检查"""
        try:
            sample_size = min(10, await self.async_redis.zcard(self.active_workers_key))
            if sample_size == 0:
                return

            random_workers = await self.async_redis.zrandmember(
                self.active_workers_key, sample_size, withscores=True
            )

            for worker_id, zset_score in random_workers:
                worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"
                hash_heartbeat = await self.async_redis.hget(worker_key, 'last_heartbeat')

                if not hash_heartbeat:
                    await self.async_redis.zrem(self.active_workers_key, worker_id)
                    logger.debug(f"Partial check: removed {worker_id}")
                else:
                    hash_time = float(hash_heartbeat)
                    if abs(hash_time - zset_score) > 1.0:
                        await self.async_redis.zadd(self.active_workers_key, {worker_id: hash_time})
                        logger.debug(f"Partial check: synced {worker_id}")

        except Exception as e:
            logger.debug(f"Partial check error: {e}")

    async def get_active_count(self) -> int:
        """获取活跃 worker 数量 - O(1)"""
        return await self.async_redis.zcard(self.active_workers_key)


__all__ = [
    'WorkerManager',
    'HeartbeatManager',
]



"""
JetTask 系统常量定义

集中管理所有系统级别的常量配置，包括：
- 内部消费者组
- 系统保留关键字
- 默认配置值
- 其他常量
"""

# ============================================================================
# 内部消费者组配置
# ============================================================================

# 内部消费者组前缀列表
# 任何以这些前缀开头的消费者组都会被视为内部消费者组，不会在用户界面显示
INTERNAL_CONSUMER_PREFIXES = [
    'pg_consumer_',      # PostgreSQL 消费者
    'webui_consumer_',   # WebUI 消费者
    'monitor_',          # 监控消费者
    'system_',           # 系统消费者
    '_internal_',        # 通用内部消费者
    '__',                # 双下划线开头的保留消费者
]

# 特定的内部消费者组名称（完全匹配）
INTERNAL_CONSUMER_NAMES = [
    'pg_consumer',       # 默认的 PostgreSQL 消费者
    'webui_consumer',    # 默认的 WebUI 消费者
    'system',            # 系统消费者
]

# ============================================================================
# Redis 键前缀和模式
# ============================================================================

# Redis 键前缀
REDIS_KEY_PREFIX = {
    'QUEUE': 'QUEUE',                    # 队列前缀
    'TASK': 'TASK',                      # 任务前缀
    'WORKER': 'WORKER',                  # Worker前缀
    'LEADER': 'leader',                  # Leader锁前缀
    'QUEUE_OFFSETS': 'QUEUE_OFFSETS',    # 队列偏移量
    'TASK_OFFSETS': 'TASK_OFFSETS',      # 任务偏移量
    'SCHEDULED': 'scheduled',            # 定时任务前缀
    'MONITOR': 'monitor',                # 监控前缀
}

# ============================================================================
# 数据库相关常量
# ============================================================================

# 分区表配置
PARTITION_TABLE_CONFIG = {
    'tasks': {
        'partition_by': 'created_at',
        'interval': 'daily',              # daily, weekly, monthly
        'retention_days': 30,             # 保留天数
    },
    'task_runs': {
        'partition_by': 'created_at',
        'interval': 'daily',
        'retention_days': 30,
    },
    'stream_backlog_monitor': {
        'partition_by': 'created_at',
        'interval': 'hourly',             # 小时级分区
        'retention_days': 7,              # 保留7天
    }
}

# ============================================================================
# 系统默认值
# ============================================================================

# 默认超时设置（秒）
DEFAULT_TIMEOUTS = {
    'task_execution': 300,        # 任务执行超时（5分钟）
    'task_ack': 60,               # 任务确认超时（1分钟）
    'worker_heartbeat': 30,       # Worker心跳超时（30秒）
    'lock_ttl': 60,               # 分布式锁TTL（60秒）
    'redis_operation': 5,         # Redis操作超时（5秒）
}

# 默认限制值
DEFAULT_LIMITS = {
    'max_retries': 3,             # 最大重试次数
    'max_pending_per_group': 1000,  # 每个消费组最大pending数
    'max_batch_size': 100,        # 最大批处理大小
    'max_prefetch': 100,          # 最大预取数量
}

# 监控和统计间隔（秒）
MONITOR_INTERVALS = {
    'backlog_monitor': 1,         # 积压监控间隔
    'health_check': 30,           # 健康检查间隔
    'namespace_check': 60,        # 命名空间检查间隔（默认）
    'stats_aggregation': 60,      # 统计聚合间隔
}

# ============================================================================
# 任务状态定义
# ============================================================================

# 任务状态优先级（数值越大，状态越"新"）
TASK_STATUS_PRIORITY = {
    'pending': 1,      # 待处理
    'running': 2,      # 运行中
    'success': 3,      # 成功（终态）
    'error': 3,        # 错误（终态）
    'failed': 3,       # 失败（终态）
    'timeout': 3,      # 超时（终态）
    'rejected': 3,     # 拒绝（终态）
    'cancelled': 3,    # 取消（终态）
}

# 终态状态列表
TASK_FINAL_STATES = ['success', 'error', 'failed', 'timeout', 'rejected', 'cancelled']

# 可重试状态列表
TASK_RETRYABLE_STATES = ['error', 'failed', 'timeout']

# ============================================================================
# 辅助函数
# ============================================================================

def is_internal_consumer(consumer_group: str) -> bool:
    """
    判断给定的消费者组是否为内部消费者组
    
    Args:
        consumer_group: 消费者组名称
        
    Returns:
        如果是内部消费者组返回 True，否则返回 False
    """
    if not consumer_group:
        return False
    
    # 转换为小写进行比较
    consumer_group_lower = consumer_group.lower()
    
    # 检查完全匹配（不区分大小写）
    for name in INTERNAL_CONSUMER_NAMES:
        if consumer_group_lower == name.lower():
            return True
    
    # 检查前缀匹配（不区分大小写）
    for prefix in INTERNAL_CONSUMER_PREFIXES:
        if consumer_group_lower.startswith(prefix.lower()):
            return True
    
    # 特殊处理：模糊匹配包含特定关键字的消费者组
    # 例如：pg_consumer 可能以不同格式出现，如 pg_consumer_YYDG_12345
    internal_keywords = [
        'pg_consumer',      # PostgreSQL 消费者的各种变体
        'webui_consumer',   # WebUI 消费者的各种变体
    ]
    
    for keyword in internal_keywords:
        if keyword in consumer_group_lower:
            return True
    
    return False


def filter_internal_consumers(consumer_groups: list) -> list:
    """
    从消费者组列表中过滤掉内部消费者组
    
    Args:
        consumer_groups: 消费者组名称列表
        
    Returns:
        过滤后的消费者组列表
    """
    return [cg for cg in consumer_groups if not is_internal_consumer(cg)]


def get_redis_key(namespace: str, key_type: str, *parts) -> str:
    """
    构建Redis键
    
    Args:
        namespace: 命名空间
        key_type: 键类型（从REDIS_KEY_PREFIX中选择）
        *parts: 其他键组成部分
        
    Returns:
        完整的Redis键
    """
    prefix = REDIS_KEY_PREFIX.get(key_type, key_type)
    key_parts = [namespace, prefix] + list(parts)
    return ':'.join(str(p) for p in key_parts if p)


def get_partition_interval_seconds(interval: str) -> int:
    """
    获取分区间隔对应的秒数
    
    Args:
        interval: 分区间隔（hourly, daily, weekly, monthly）
        
    Returns:
        对应的秒数
    """
    intervals = {
        'hourly': 3600,
        'daily': 86400,
        'weekly': 604800,
        'monthly': 2592000,  # 30天
    }
    return intervals.get(interval, 86400)  # 默认返回一天


# -*- coding: utf-8 -*-
"""
JetTask - High Performance Distributed Task Queue System
"""

import logging
import inspect

# Core class imports
from jettask.core.app import Jettask
from jettask.core.message import TaskMessage
from jettask.core.context import TaskContext
from jettask.task.task_center.client import TaskCenter
from jettask.task.router import TaskRouter
from jettask.scheduler.schedule import Schedule

# Rate limit config imports
from jettask.utils.rate_limit.config import QPSLimit, ConcurrencyLimit

# Import logger components from utils
from jettask.utils.task_logger import (
    TaskContextFilter,
    ExtendedTextFormatter,
    LogContext
)

# Version info
__version__ = "0.1.0"


def get_task_logger(name: str = None) -> logging.Logger:
    """
    获取带任务上下文的logger
    
    Args:
        name: logger名称，默认使用调用者的模块名
    
    Returns:
        配置好的logger实例
        
    Example:
        from jettask import get_task_logger
        logger = get_task_logger()
        logger.info("处理任务")
    """
    if name is None:
        # 自动获取调用者的模块名
        frame = inspect.currentframe()
        if frame and frame.f_back:
            name = frame.f_back.f_globals.get('__name__', 'jettask')
        else:
            name = 'jettask'
    
    logger = logging.getLogger(name)
    
    # 如果logger还没有处理器，添加默认处理器
    if not logger.handlers:
        handler = logging.StreamHandler()
        handler.setFormatter(ExtendedTextFormatter(
            '%(asctime)s - %(levelname)s - [%(task_id)s] - %(name)s - %(message)s'
        ))
        handler.addFilter(TaskContextFilter())
        logger.addHandler(handler)
        logger.propagate = False
    
    return logger


# Public API exports
__all__ = [
    "Jettask",
    "TaskMessage",
    "Schedule",
    "TaskCenter",
    "TaskRouter",
    "QPSLimit",
    "ConcurrencyLimit",
    "get_task_logger",
    "LogContext",
    "TaskContext"
]


import logging
from contextlib import asynccontextmanager
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
import uvicorn

from jettask.webui.services import MonitorService

logger = logging.getLogger(__name__)

# 创建全局监控器实例
monitor = MonitorService()

@asynccontextmanager
async def lifespan(app: FastAPI):

    # Startup
    try:
        import os
        # 检查是否使用Nacos配置
        use_nacos = os.getenv('USE_NACOS', 'false').lower() == 'true'

        # 直接使用 connector.py 管理数据库连接
        from jettask.config.task_center import task_center_config

        # 存储配置信息在 app.state 中，供路由使用
        app.state.redis_url = os.environ.get('JETTASK_REDIS_URL', 'redis://localhost:6379/0')
        app.state.pg_url = os.environ.get('JETTASK_PG_URL', 'postgresql+asyncpg://jettask:123456@localhost:5432/jettask')
        app.state.redis_prefix = os.environ.get('JETTASK_REDIS_PREFIX', 'jettask')

        # 记录任务中心配置
        logger.info("=" * 60)
        logger.info("任务中心配置:")
        logger.info(f"  配置模式: {'Nacos' if use_nacos else '环境变量'}")
        logger.info(f"  元数据库: {task_center_config.meta_db_host}:{task_center_config.meta_db_port}/{task_center_config.meta_db_name}")
        logger.info(f"  API服务: {task_center_config.api_host}:{task_center_config.api_port}")
        logger.info(f"  基础URL: {task_center_config.base_url}")
        logger.info("=" * 60)

        # 连接 monitor
        # await monitor.connect()
        # # 启动心跳扫描器
        # await monitor.start_heartbeat_scanner()
        # # 将 monitor 存储到 app.state 供新路由使用
        # app.state.monitor = monitor

        # PostgreSQL consumer 已弃用，由统一的数据库管理器处理
        logging.info("PostgreSQL consumer disabled (use --with-consumer to enable)")

        logger.info("JetTask WebUI 启动成功")
    except Exception as e:
        logger.error(f"启动失败: {e}")
        import traceback
        traceback.print_exc()
        raise

    yield

    # Shutdown
    try:
        # 停止心跳扫描器
        await monitor.stop_heartbeat_scanner()
        await monitor.close()

        # 数据库连接池由 connector.py 全局管理
        # 不需要显式关闭，它们会在进程结束时自动清理

        logger.info("JetTask WebUI 关闭完成")
    except Exception as e:
        logger.error(f"关闭时出错: {e}")
        import traceback
        traceback.print_exc()

app = FastAPI(title="Jettask Monitor", lifespan=lifespan)

# 配置 CORS
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # 允许所有来源（生产环境应该指定具体域名）
    allow_credentials=True,
    allow_methods=["*"],  # 允许所有 HTTP 方法
    allow_headers=["*"],  # 允许所有请求头
)

# 配置 Namespace 自动注入中间件
# 这个中间件会自动检测路由中的 {namespace} 参数，并注入到 request.state.ns
from jettask.webui.middleware import NamespaceMiddleware
app.add_middleware(NamespaceMiddleware)
logger.info("NamespaceMiddleware 已注册 - 所有包含 {namespace} 的路由将自动注入命名空间上下文")

# 注册 API 路由
from jettask.webui.api import api_router
app.include_router(api_router)

# ============ WebSocket 实时推送 ============
# （parse_time_duration 已移除，因为 PG 时间轴路由已迁移）

# ============ 已迁移路由 ============
# 以下路由已迁移到模块化的 API 路由：
# - GET /api/queues → api/queues.py
# - GET /api/queue/{queue_name}/stats → api/queues.py
# - GET /api/queue/{queue_name}/workers → api/workers.py
# - GET /api/queue/{queue_name}/worker-summary → api/workers.py
# - GET /api/workers/offline-history → api/workers.py
# - GET /api/global-stats → api/overview.py
# - GET /api/global-stats/light → api/overview.py
# ====================================

# GET /api/queue/{queue_name}/workers/offline-history → 已迁移到 api/workers.py


# ============ PostgreSQL 路由已迁移 ============
# 以下路由已迁移到 api/analytics.py：
# - GET /api/pg/tasks → GET /api/v1/analytics/pg/tasks
# - GET /api/pg/stats → GET /api/v1/analytics/pg/stats
# - GET /api/pg/task/{task_id} → GET /api/v1/analytics/pg/task/{task_id}
# ==============================================

if __name__ == "__main__":
    # 配置日志
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )
    
    uvicorn.run(app, host="0.0.0.0", port=8000)


"""
业务逻辑服务层
包含所有业务逻辑处理
"""

from .overview_service import OverviewService
from .queue_service import QueueService
from .scheduled_task_service import ScheduledTaskService
from .alert_service import AlertService
from .analytics_service import AnalyticsService
from .settings_service import SettingsService
from .task_service import TaskService

# 监控服务
from .redis_monitor_service import RedisMonitorService
from .task_monitor_service import TaskMonitorService
from .worker_monitor_service import WorkerMonitorService
from .queue_monitor_service import QueueMonitorService
from .heartbeat_service import HeartbeatService
from .timeline_service import TimelineService
from .timeline_pg_service import TimelinePgService


class MonitorService:
    """
    统一的监控服务类

    整合所有监控服务，提供统一的接口
    """

    def __init__(self, redis_url: str = "redis://localhost:6379", redis_prefix: str = "jettask"):
        """
        初始化监控服务

        Args:
            redis_url: Redis 连接 URL
            redis_prefix: Redis 键前缀
        """
        # 创建基础 Redis 服务
        self.redis_service = RedisMonitorService(redis_url, redis_prefix)

        # 创建各个子服务
        self.task_service = TaskMonitorService(self.redis_service)
        self.worker_service = WorkerMonitorService(self.redis_service)
        self.queue_service = QueueMonitorService(self.redis_service)
        self.heartbeat_service = HeartbeatService(self.redis_service)
        self.timeline_service = TimelineService(self.redis_service)

    async def connect(self):
        """连接到 Redis"""
        await self.redis_service.connect()

    async def close(self):
        """关闭所有服务"""
        # 停止心跳扫描器
        await self.heartbeat_service.stop_heartbeat_scanner()
        # 关闭 Redis 连接
        await self.redis_service.close()

    async def __aenter__(self):
        """异步上下文管理器入口"""
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器退出"""
        await self.close()

    # ==================== Task Monitor Methods ====================

    async def get_task_info(self, stream_id: str, queue_name: str):
        """获取单个任务的详细信息"""
        return await self.task_service.get_task_info(stream_id, queue_name)

    async def get_stream_info(self, queue_name: str):
        """获取 Stream 的统计信息"""
        return await self.task_service.get_stream_info(queue_name)

    async def get_queue_tasks(self, queue_name: str, start: str = "-", end: str = "+", count: int = 100, reverse: bool = False):
        """获取队列中的任务列表"""
        return await self.task_service.get_queue_tasks(queue_name, start, end, count, reverse)

    # ==================== Worker Monitor Methods ====================

    async def get_worker_heartbeats(self, queue_name: str):
        """获取指定队列的 Worker 心跳信息"""
        return await self.worker_service.get_worker_heartbeats(queue_name)

    async def get_queue_worker_summary(self, queue_name: str):
        """获取队列的 Worker 汇总统计信息（包含历史数据）"""
        return await self.worker_service.get_queue_worker_summary(queue_name)

    async def get_queue_worker_summary_fast(self, queue_name: str):
        """获取队列的 Worker 汇总统计信息（快速版，仅在线 Worker）"""
        return await self.worker_service.get_queue_worker_summary_fast(queue_name)

    async def get_worker_offline_history(self, limit: int = 100, start_time=None, end_time=None):
        """获取 Worker 下线历史记录"""
        return await self.worker_service.get_worker_offline_history(limit, start_time, end_time)

    # ==================== Queue Monitor Methods ====================

    async def get_all_queues(self):
        """获取所有队列名称"""
        return await self.queue_service.get_all_queues()

    async def get_queue_stats(self, queue_name: str):
        """获取队列统计信息（RabbitMQ 兼容格式）"""
        return await self.queue_service.get_queue_stats(queue_name)

    # ==================== Heartbeat Service Methods ====================

    async def start_heartbeat_scanner(self):
        """启动心跳扫描器"""
        await self.heartbeat_service.start_heartbeat_scanner()

    async def stop_heartbeat_scanner(self):
        """停止心跳扫描器"""
        await self.heartbeat_service.stop_heartbeat_scanner()

    async def check_worker_heartbeat(self, worker_id: str):
        """检查单个 Worker 的心跳状态"""
        return await self.heartbeat_service.check_worker_heartbeat(worker_id)

    async def get_heartbeat_stats(self):
        """获取心跳监控统计信息"""
        return await self.heartbeat_service.get_heartbeat_stats()

    # ==================== Timeline Service Methods ====================

    async def get_redis_timeline(self, queue_name: str, **kwargs):
        """获取 Redis Stream 时间轴数据"""
        return await self.timeline_service.get_redis_timeline(queue_name, **kwargs)

    # ==================== Utility Methods ====================

    def get_prefixed_queue_name(self, queue_name: str) -> str:
        """为队列名称添加前缀"""
        return self.redis_service.get_prefixed_queue_name(queue_name)

    @property
    def redis(self):
        """获取 Redis 客户端"""
        return self.redis_service.redis

    @property
    def redis_prefix(self) -> str:
        """获取 Redis 前缀"""
        return self.redis_service.redis_prefix


__all__ = [
    'OverviewService',
    'QueueService',
    'ScheduledTaskService',
    'AlertService',
    'AnalyticsService',
    'SettingsService',
    'TaskService',
    'MonitorService',
    'RedisMonitorService',
    'TaskMonitorService',
    'WorkerMonitorService',
    'QueueMonitorService',
    'HeartbeatService',
    'TimelineService',
    'TimelinePgService',
]


"""
简化的离线worker消息恢复模块
"""
import asyncio
import json
import logging
from typing import Dict, List, Optional, Tuple, TYPE_CHECKING
from redis.asyncio.lock import Lock as AsyncLock

import msgpack

if TYPE_CHECKING:
    from jettask.worker.lifecycle import WorkerManager

logger = logging.getLogger(__name__)


class OfflineWorkerRecovery:
    """离线worker消息恢复处理器"""

    def __init__(self, async_redis_client, redis_prefix='jettask', worker_prefix='WORKER', queue_formatter=None, queue_registry=None, worker_state: Optional['WorkerManager'] = None, app=None):
        """
        初始化离线worker消息恢复处理器

        Args:
            async_redis_client: 异步Redis客户端
            redis_prefix: Redis键前缀
            worker_prefix: Worker键前缀
            queue_formatter: 队列格式化函数
            queue_registry: 队列注册表
            worker_state: WorkerState实例（用于查询Worker状态）
            app: App实例（用于访问worker_state_manager和worker_id）
        """
        self.async_redis_client = async_redis_client
        self.app = app
        self.redis_prefix = redis_prefix
        self.worker_prefix = worker_prefix
        self._stop_recovery = False
        # 队列格式化函数，默认使用 prefix:QUEUE:queue_name 格式
        self.queue_formatter = queue_formatter or (lambda q: f"{self.redis_prefix}:QUEUE:{q}")
        # 通过 app 访问 worker_state_manager
        self.worker_state_manager = app.worker_state_manager if (app and hasattr(app, 'worker_state_manager')) else None
        # 队列注册表，用于获取优先级队列
        self.queue_registry = queue_registry
        # Worker状态查询器（必须传入）
        self._worker_state: Optional['WorkerState'] = worker_state
        
    async def recover_offline_workers(self,
                                     queue: str,
                                     task_name: str,
                                     current_consumer_name: str = None,
                                     event_queue: Optional[asyncio.Queue] = None,
                                     event_queue_callback: Optional[callable] = None) -> int:
        """
        恢复指定任务的离线worker的pending消息

        Args:
            queue: 队列名（可以是优先级队列）
            task_name: 任务名称
            current_consumer_name: 当前消费者名称
            event_queue: 事件队列
            event_queue_callback: 获取任务事件队列的回调函数

        Returns:
            恢复的消息数量
        """
        total_recovered = 0
        logger.debug(f'恢复任务 {task_name} 在队列 {queue} 的离线worker的pending消息')

        try:
            # 直接查找该任务的离线 worker
            offline_workers = await self._find_offline_workers(task_name)
            if not offline_workers:
                logger.info(f"No offline workers found for task {task_name}")
                return 0

            logger.info(f"Found {len(offline_workers)} offline workers for task {task_name}, starting recovery...")

            # 确定基础队列名（用于传递给恢复逻辑）
            base_queue = queue.split(':')[0]

            # 使用辅助方法恢复（只恢复传入的队列）
            total_recovered = await self._recover_queues_for_workers(
                [queue],  # 只恢复当前队列
                offline_workers,
                base_queue,
                current_consumer_name,
                event_queue,
                event_queue_callback
            )

        except Exception as e:
            logger.error(f"Error recovering offline workers for queue {queue}: {e}")
            import traceback
            traceback.print_exc()

        return total_recovered
        
    async def _find_offline_workers(self, task_name: str) -> List[Tuple[str, Dict]]:
        """查找指定任务的离线worker

        委托给 WorkerManager.find_offline_workers_for_task() 方法

        注意：worker_state 必须在 OfflineWorkerRecovery 初始化时传入
        """
        if self._worker_state is None:
            raise RuntimeError(
                "WorkerState not provided to OfflineWorkerRecovery. "
                "Please pass worker_state parameter during initialization."
            )

        return await self._worker_state.find_offline_workers_for_task(
            task_name=task_name,
            worker_prefix=self.worker_prefix
        )

    async def _recover_queues_for_workers(
        self,
        queues_to_recover: List[str],
        offline_workers: List[Tuple[str, Dict]],
        base_queue: str,
        current_consumer_name: str = None,
        event_queue: Optional[asyncio.Queue] = None,
        event_queue_callback: Optional[callable] = None
    ) -> int:
        """
        恢复一组队列的离线 worker 消息

        Args:
            queues_to_recover: 需要恢复的队列列表（包括优先级队列）
            offline_workers: 离线 worker 列表
            base_queue: 基础队列名
            其他参数同 recover_offline_workers_for_queue

        Returns:
            恢复的消息总数
        """
        total_recovered = 0

        # 🔧 获取当前 worker 的所有 group_name（用于匹配检查）
        current_worker_group_names = set()
        if self.app and self.app.worker_id:
            current_worker_key = f"{self.redis_prefix}:WORKER:{self.app.worker_id}"
            current_worker_data = await self.async_redis_client.hgetall(current_worker_key)

            for key, value in current_worker_data.items():
                if isinstance(key, bytes):
                    key = key.decode('utf-8')
                if isinstance(value, bytes):
                    value = value.decode('utf-8')

                if key.startswith('group_info:'):
                    try:
                        group_info = json.loads(value)
                        group_name = group_info.get('group_name')
                        if group_name:
                            current_worker_group_names.add(group_name)
                    except Exception as e:
                        logger.error(f"Error parsing current worker group_info: {e}")

            logger.info(f"[Recovery] 当前 worker 的 group_names: {current_worker_group_names}")
        else:
            logger.warning(f"[Recovery] 无法获取当前 worker 的 group_names (app.worker_id not available)")

        # 获取当前 worker 运行的任务名称集合
        current_task_names = set()
        if self.app and hasattr(self.app, '_tasks'):
            current_task_names = set(self.app._tasks.keys())
            logger.info(f"[Recovery] 当前 worker 运行的任务: {current_task_names}")
        else:
            logger.warning(f"[Recovery] 无法获取当前 worker 的任务列表，将恢复所有匹配队列的消息")

        # 对每个离线worker，提前提取 group_infos（使用任务名称匹配）
        workers_with_groups = []
        for worker_key, worker_data in offline_workers:
            group_infos = []
            for key, value in worker_data.items():
                if key.startswith('group_info:'):
                    try:
                        group_info = json.loads(value)
                        task_name = group_info.get('task_name')
                        group_queue = group_info.get('queue')

                        # 新的匹配逻辑：按任务名称匹配
                        # 1. 如果有当前任务列表，只恢复当前运行的任务的消息
                        # 2. 如果没有当前任务列表，则恢复所有消息（兼容旧逻辑）
                        if current_task_names:
                            if task_name in current_task_names:
                                group_infos.append(group_info)
                                logger.debug(f"Found group info for task {task_name} (task name match): {group_info}")
                            else:
                                logger.debug(f"Skipping group info for task {task_name} (not in current tasks)")
                        else:
                            # fallback: 如果无法获取当前任务列表，使用队列精确匹配逻辑
                            if group_queue == base_queue:
                                group_infos.append(group_info)
                                logger.debug(f"Found group info for base queue {base_queue} (exact match): {group_info}")
                    except Exception as e:
                        logger.error(f"Error parsing group_info: {e}")

            workers_with_groups.append((worker_key, worker_data, group_infos))
            logger.info(f"Worker {worker_key} has {len(group_infos)} groups (filtered by task names)")

        # 对每个队列（基础队列 + 优先级队列）恢复消息
        for queue_to_recover in queues_to_recover:
            if self._stop_recovery:
                logger.debug("Stopping recovery due to shutdown signal")
                break

            logger.info(f"Recovering queue: {queue_to_recover}")

            # 获取当前consumer名称
            consumer_name = current_consumer_name
            if not consumer_name and self.app and self.app.worker_id:
                # 🔧 修复：从当前worker的group_info中查找匹配的consumer_name
                # 而不是硬编码 {worker_id}-{base_queue}
                current_worker_key = f"{self.redis_prefix}:WORKER:{self.app.worker_id}"
                current_worker_data = await self.async_redis_client.hgetall(current_worker_key)

                # 查找匹配base_queue的group_info（精确匹配）
                found_consumer_name = None

                for key, value in current_worker_data.items():
                    if isinstance(key, bytes):
                        key = key.decode('utf-8')
                    if isinstance(value, bytes):
                        value = value.decode('utf-8')

                    if key.startswith('group_info:'):
                        try:
                            current_group_info = json.loads(value)
                            current_group_queue = current_group_info.get('queue')

                            # 精确匹配
                            if current_group_queue == base_queue:
                                found_consumer_name = current_group_info.get('consumer_name')
                                logger.debug(f"Found current consumer_name by exact match: {found_consumer_name}")
                                break
                        except Exception as e:
                            logger.error(f"Error parsing current group_info: {e}")

                if found_consumer_name:
                    consumer_name = found_consumer_name
                else:
                    # fallback：构造默认的consumer_name
                    consumer_name = f"{self.app.worker_id}-{base_queue}"
                    logger.warning(f"Could not find consumer_name in group_info, using fallback: {consumer_name}")

            if not consumer_name:
                logger.error(f"Cannot get current consumer name for queue {queue_to_recover}")
                continue

            logger.info(f"Starting recovery for queue {queue_to_recover} with consumer {consumer_name}")

            # 处理每个离线worker的这个队列的消息（传入预提取的 group_infos）
            for worker_key, worker_data, group_infos in workers_with_groups:
                if self._stop_recovery:
                    logger.debug("Stopping recovery due to shutdown signal")
                    break

                logger.info(f"Recovering messages from worker {worker_key} for queue {queue_to_recover}")
                recovered = await self._recover_worker_messages(
                    queue=queue_to_recover,
                    worker_key=worker_key,
                    worker_data=worker_data,
                    group_infos=group_infos,
                    current_consumer_name=consumer_name,
                    event_queue=event_queue,
                    event_queue_callback=event_queue_callback,
                    current_worker_group_names=current_worker_group_names  # 🔧 传递当前 worker 的 group_names
                )

                total_recovered += recovered

        return total_recovered

    async def _recover_worker_messages(self,
                                      queue: str,
                                      worker_key: str,
                                      worker_data: Dict,
                                      group_infos: List[Dict],
                                      current_consumer_name: str,
                                      event_queue: Optional[asyncio.Queue] = None,
                                      event_queue_callback: Optional[callable] = None,
                                      current_worker_group_names: Optional[set] = None) -> int:
        """
        恢复单个worker的pending消息

        Args:
            queue: 当前要恢复的队列（可能是基础队列或优先级队列）
            worker_key: Worker的Redis键
            worker_data: Worker的数据
            group_infos: 预提取的group_info列表（已按基础队列过滤）
            current_consumer_name: 当前consumer名称
            event_queue: 事件队列
            event_queue_callback: 获取任务事件队列的回调函数
            current_worker_group_names: 当前 worker 的所有 group_name 集合（用于匹配检查）

        Returns:
            恢复的消息数量
        """
        total_claimed = 0
        transferred = False
        # 提取 worker_id 用于分布式锁和标记
        worker_id = worker_key.split(':')[-1]

        # 🔧 使用分布式锁防止多个 worker 同时恢复同一个离线 worker 的消息
        recovery_lock_key = f"{self.redis_prefix}:RECOVERY:LOCK:{worker_id}:{queue}"
        recovery_lock = AsyncLock(
            self.async_redis_client,
            recovery_lock_key,
            timeout=60,  # 恢复可能需要较长时间
            blocking=False
        )

        if not await recovery_lock.acquire():
            logger.info(f"[Recovery] 恢复锁被占用，跳过: worker={worker_id}, queue={queue}")
            return 0

        try:
            # 🔧 检查是否已经转移过消息（避免重复恢复）
            messages_transferred = worker_data.get('messages_transferred', 'false')
            if isinstance(messages_transferred, bytes):
                messages_transferred = messages_transferred.decode('utf-8')

            if messages_transferred.lower() == 'true':
                logger.info(f"[Recovery] 消息已转移，跳过: worker={worker_id}, queue={queue}")
                return 0

            if not group_infos:
                logger.info(f"[Recovery] 没有 group_info，跳过: worker={worker_id}, queue={queue}")
                return 0

            # 处理每个group_info
            for group_info in group_infos:
                base_stream_key = group_info.get('stream_key')
                group_name = group_info.get('group_name')
                base_offline_consumer_name = group_info.get('consumer_name')
                task_name = group_info.get('task_name')
                base_queue = group_info.get('queue')

                if not all([base_stream_key, group_name, base_offline_consumer_name]):
                    logger.warning(f"Incomplete group_info: {group_info}")
                    continue

                # 🔧 检查当前 worker 是否有相同的 group_name
                if current_worker_group_names is not None and group_name not in current_worker_group_names:
                    logger.info(
                        f"[Recovery] 跳过恢复: group_name '{group_name}' 不在当前 worker 的 group_names 中 "
                        f"(worker: {worker_key}, task: {task_name})"
                    )
                    continue

                logger.info(f"[Recovery] ✓ group_name 匹配: {group_name}, 开始恢复消息")

                # 🔧 检查队列是否匹配：group_info 中的队列必须与当前要恢复的队列匹配
                # 直接比较完整队列名（包括优先级后缀）
                if base_queue != queue:
                    logger.debug(
                        f"[Recovery] 跳过恢复: 队列不匹配 "
                        f"(group_info queue={base_queue}, current queue={queue}, task={task_name})"
                    )
                    continue

                # 构建 stream_key 和 offline_consumer_name
                # group_info 中存储的是完整队列名（包括优先级后缀，如 robust_bench2:8）
                stream_key = f"{self.redis_prefix}:QUEUE:{queue}"
                offline_consumer_name = base_offline_consumer_name

                logger.info(f"Recovering task {task_name}: stream={stream_key}, group={group_name}, consumer={offline_consumer_name}")

                # 跳过自己的consumer，但只有在worker仍然活跃的情况下
                # 如果worker已经offline（is_alive=false或messages_transferred=true），即使consumer名称相同也应该恢复
                # 这处理了worker_id被复用的情况
                is_alive = worker_data.get('is_alive', 'false')
                if isinstance(is_alive, bytes):
                    is_alive = is_alive.decode('utf-8')
                is_alive = is_alive.lower() == 'true'

                messages_transferred = worker_data.get('messages_transferred', 'false')
                if isinstance(messages_transferred, bytes):
                    messages_transferred = messages_transferred.decode('utf-8')
                messages_transferred = messages_transferred.lower() == 'true'

                # 只有在worker活跃且消息未转移时，才跳过同名consumer
                if current_consumer_name == offline_consumer_name and is_alive and not messages_transferred:
                    logger.info(f"Skipping own active consumer: {offline_consumer_name}")
                    continue
                elif current_consumer_name == offline_consumer_name:
                    logger.info(f"Recovering same-name consumer from offline worker: {offline_consumer_name} (is_alive={is_alive}, messages_transferred={messages_transferred})")

                # 🔧 使用分布式锁保护单个 consumer group 的恢复（细粒度锁）
                # 这个锁与外层的 worker 级别锁不同，防止多个进程同时恢复同一个 consumer group
                lock_key = f"{self.redis_prefix}:CLAIM:LOCK:{offline_consumer_name}:{group_name}"
                group_lock = AsyncLock(
                    self.async_redis_client,
                    lock_key,
                    timeout=30,
                    blocking=False
                )

                if not await group_lock.acquire():
                    logger.info(f"[Recovery] consumer group 锁被占用，跳过: {offline_consumer_name}:{group_name}")
                    continue

                try:
                    # 🔧 优化：直接使用 xpending_range 获取 pending 消息（不需要先调用 xpending）
                    # 因为所有消息都通过 consumer group 消费，直接从 pending 列表恢复即可
                    batch_size = 100
                    total_claimed_count = 0

                    # 循环直到处理完所有消息
                    while True:
                        # 获取具体的pending消息信息（每次最多 batch_size 条）
                        detailed_pending = await self.async_redis_client.xpending_range(
                            stream_key, group_name,
                            min='-', max='+', count=batch_size,
                            consumername=offline_consumer_name
                        )

                        if not detailed_pending:
                            # 没有更多消息了
                            break

                        # 标记有消息被转移（即使最终没有成功 claim）
                        if not transferred:
                            transferred = True

                        logger.info(f"Found {len(detailed_pending)} pending messages for {task_name} (batch {total_claimed_count // batch_size + 1})")

                        # 批量认领消息
                        message_ids = [msg['message_id'] for msg in detailed_pending]
                        claimed_messages = await self.async_redis_client.xclaim(
                            stream_key, group_name,
                            current_consumer_name,
                            min_idle_time=0,
                            message_ids=message_ids
                        )

                        if claimed_messages:
                            logger.info(f"Claimed {len(claimed_messages)} messages for task {task_name} in this batch")
                            total_claimed_count += len(claimed_messages)

                            # 获取该任务的 event_queue
                            # 优先使用 event_queue_callback，其次使用直接传入的 event_queue
                            task_event_queue = None
                            if event_queue_callback and task_name:
                                task_event_queue = event_queue_callback(task_name)
                            elif event_queue:
                                task_event_queue = event_queue

                            # 如果有 event_queue，将消息放入队列
                            if task_event_queue:
                                logger.info(f'即将转移 {len(claimed_messages)=} 消息到 {task_name}')
                                for msg_id, msg_data in claimed_messages:
                                    if isinstance(msg_id, bytes):
                                        msg_id = msg_id.decode('utf-8')

                                    # 解析消息数据
                                    data_field = msg_data.get(b'data') or msg_data.get('data')
                                    if data_field:
                                        try:
                                            parsed_data = msgpack.unpackb(data_field, raw=False)
                                            # 添加必要的元数据
                                            parsed_data['_task_name'] = task_name
                                            parsed_data['queue'] = queue

                                            # 构建任务项
                                            task_item = {
                                                'queue': queue,
                                                'event_id': msg_id,
                                                'event_data': parsed_data,
                                                'consumer': current_consumer_name,
                                                'group_name': group_name
                                            }

                                            await task_event_queue.put(task_item)
                                            logger.debug(f"Put recovered message {msg_id} into event_queue for task {task_name}")
                                        except Exception as e:
                                            logger.error(f"Error processing claimed message: {e}")
                            else:
                                logger.warning(f"No event_queue available for task {task_name}, claimed messages will not be executed")

                            # 更新总计数
                            total_claimed += len(claimed_messages)

                            # 如果这批处理的消息数少于 batch_size，说明已经处理完了
                            if len(detailed_pending) < batch_size:
                                break
                        else:
                            # 没有成功 claim 到消息，退出循环
                            break

                        # 记录总恢复数量
                        if total_claimed_count > 0:
                            logger.info(f"Total claimed {total_claimed_count} messages for {task_name} from {offline_consumer_name}")
                finally:
                    await group_lock.release()

            # 🔧 所有消息恢复完成后，标记该 worker 的消息已转移
            if total_claimed > 0 or transferred:
                logger.info(f"[Recovery] 成功恢复 {total_claimed} 条消息，标记 worker {worker_id} 消息已转移")
                if self.worker_state_manager:
                    await self.worker_state_manager.mark_messages_transferred(worker_id, transferred=True)
                else:
                    await self.async_redis_client.hset(worker_key, 'messages_transferred', 'true')
            else:
                logger.info(f"[Recovery] 没有消息需要恢复: worker={worker_id}, queue={queue}")

        except Exception as e:
            import traceback
            traceback.print_exc()
            logger.error(f"[Recovery] 恢复消息时出错: {e}")

        finally:
            # 🔧 释放分布式锁
            await recovery_lock.release()

        return total_claimed
        
    async def _get_consumer_groups(self, stream_key: str, suffix: Optional[str] = None) -> List[str]:
        """获取Stream的consumer groups"""
        groups = []
        try:
            # 确保stream_key是字符串类型（如果是bytes会出问题）
            if isinstance(stream_key, bytes):
                stream_key = stream_key.decode('utf-8')
            
            all_groups = await self.async_redis_client.xinfo_groups(stream_key)
            logger.debug(f"Raw groups info for {stream_key}: {all_groups}")
            
            for group_info in all_groups:
                # 二进制Redis客户端返回的字典键是字符串，值是bytes
                group_name = group_info.get('name', b'')
                logger.debug(f"Processing group: {group_info}, group_name type: {type(group_name)}")
                
                # 解码group名称
                if isinstance(group_name, bytes):
                    group_name = group_name.decode('utf-8')
                
                # 过滤空的group名称
                if group_name:
                    if suffix:
                        if group_name.endswith(suffix):
                            groups.append(group_name)
                    else:
                        groups.append(group_name)
                        logger.debug(f"Added group: {group_name}")
        except Exception as e:
            logger.error(f"Error getting consumer groups for {stream_key}: {e}")
        return groups
        
    async def _claim_messages(self, stream_key: str, group_name: str, 
                             old_consumer: str, new_consumer: str) -> List[Tuple[bytes, Dict]]:
        """转移pending消息"""
        all_claimed = []
        last_id = '-'
        
        try:
            # 确保参数是bytes类型
            if isinstance(stream_key, str):
                stream_key = stream_key.encode('utf-8')
            if isinstance(group_name, str):
                group_name = group_name.encode('utf-8')
            
            logger.debug(f"_claim_messages: stream_key={stream_key}, group_name={group_name}, old_consumer={old_consumer}, new_consumer={new_consumer}")
                
            while True:
                # 获取pending消息
                pending_batch = await self.async_redis_client.xpending_range(
                    stream_key, group_name,
                    min=last_id, max='+',
                    count=100
                )
                
                logger.debug(f"Got {len(pending_batch) if pending_batch else 0} pending messages")
                
                if not pending_batch:
                    break
                
                # 过滤出属于旧consumer的消息
                message_ids = []
                for msg in pending_batch:
                    msg_consumer = msg.get('consumer') or msg.get(b'consumer')
                    if isinstance(msg_consumer, bytes):
                        msg_consumer = msg_consumer.decode('utf-8')
                    
                    if msg_consumer == old_consumer:
                        msg_id = msg.get('message_id') or msg.get(b'message_id')
                        message_ids.append(msg_id)
                
                if message_ids:
                    # 使用XCLAIM转移消息
                    logger.debug(f"Claiming {len(message_ids)} messages from {old_consumer} to {new_consumer}")
                    
                    claimed = await self.async_redis_client.xclaim(
                        stream_key, group_name,
                        new_consumer,
                        min_idle_time=0,
                        message_ids=message_ids,
                        force=True
                    )
                    
                    if claimed:
                        all_claimed.extend(claimed)
                
                # 更新游标
                if pending_batch:
                    last_msg_id = pending_batch[-1].get('message_id') or pending_batch[-1].get(b'message_id')
                    if isinstance(last_msg_id, bytes):
                        last_msg_id = last_msg_id.decode('utf-8')
                    # 增加ID以获取下一批
                    parts = last_msg_id.split('-')
                    if len(parts) == 2:
                        last_id = f"{parts[0]}-{int(parts[1]) + 1}"
                    else:
                        break
                else:
                    break
                    
        except Exception as e:
            logger.error(f"Error claiming messages: {e}")
            
        return all_claimed
        
    async def _put_to_event_queue(self, msg_id, msg_data, queue, event_queue, 
                                 consumer, group_name, old_consumer):
        """将转移的消息放入event_queue"""
        try:
            # 解析消息数据
            if b'data' in msg_data:
                event_data = msgpack.unpackb(msg_data[b'data'], raw=False)
            else:
                event_data = msg_data
            
            # 从 group_name 中提取 task_name
            # group_name 的格式是: "jettask:QUEUE:{queue}:{task_name}"
            task_name = None
            if group_name and ':' in group_name:
                parts = group_name.split(':')
                # 查找最后一个非数字部分作为task_name
                for i in range(len(parts) - 1, -1, -1):
                    part = parts[i]
                    # 跳过优先级数字
                    if not part.isdigit() and part not in ['jettask', 'QUEUE', queue]:
                        task_name = part
                        logger.debug(f"Extracted task_name '{task_name}' from group_name '{group_name}'")
                        break
            
            # 如果从group_name提取失败，尝试从consumer名称提取
            if not task_name and ':' in consumer and ':' in group_name:
                # consumer格式可能是: "{consumer_id}:{task_name}" 
                consumer_parts = consumer.split(':')
                if len(consumer_parts) > 1:
                    potential_task = consumer_parts[-1]
                    # 确保不是优先级数字
                    if not potential_task.isdigit():
                        task_name = potential_task
                        logger.debug(f"Extracted task_name '{task_name}' from consumer '{consumer}'")
            
            # 如果还是没有task_name，检查event_data中是否已有
            if not task_name and '_task_name' in event_data:
                task_name = event_data['_task_name']
                logger.debug(f"Using existing _task_name from event_data: '{task_name}'")
            
            # 确保event_data中有_task_name字段
            if task_name:
                event_data['_task_name'] = task_name
                logger.debug(f"Added _task_name '{task_name}' to recovered message")
            else:
                # 如果无法确定task_name，记录警告
                logger.warning(f"Could not determine task_name for recovered message. group_name='{group_name}', consumer='{consumer}'")
            
            # 构建事件
            event = {
                'event_id': msg_id.decode() if isinstance(msg_id, bytes) else msg_id,
                'event_data': event_data,
                'queue': queue,
                'consumer': consumer,
                'group_name': group_name,
                '_recovery': True,
                '_claimed_from': old_consumer
            }
            
            await event_queue.put(event)
            
        except Exception as e:
            logger.error(f"Error putting message to event queue: {e}")
            
    def stop(self):
        """停止恢复处理"""
        self._stop_recovery = True


"""
队列统计API v2 - 支持消费者组和优先级队列
"""
import asyncio
import logging
from typing import Dict, List, Optional, Any
from datetime import datetime, timezone
from collections import defaultdict
import redis.asyncio as redis
from sqlalchemy import text
from sqlalchemy.ext.asyncio import AsyncSession
from jettask.config.constants import is_internal_consumer

logger = logging.getLogger(__name__)


class QueueStatsV2:
    """队列统计服务V2 - 支持消费者组详情"""
    
    def __init__(self, redis_client: redis.Redis, pg_session: Optional[AsyncSession] = None, redis_prefix: str = "default"):
        self.redis_client = redis_client
        self.pg_session = pg_session
        self.redis_prefix = redis_prefix
    
    async def get_queue_stats_grouped(self, time_filter: Optional[Dict[str, Any]] = None) -> List[dict]:
        """
        获取按队列分组的统计信息（前端使用的格式）
        返回格式：[{
            'queue_name': 'queue1',
            'consumer_groups': [...],
            'consumer_groups_count': 3,
            'total_length': 1000,
            'priority_queues': [0, 1, 2],
            ...
        }]
        """
        # 先获取扁平化的数据
        flat_stats = await self.get_queue_stats_with_groups(time_filter)
        
        # 转换为分组格式
        return await self._transform_to_grouped_format(flat_stats)
    
    async def get_queue_stats_with_groups(self, time_filter: Optional[Dict[str, Any]] = None) -> List[dict]:
        """
        获取扁平化的队列统计信息 - 优化版本
        
        Args:
            time_filter: 时间筛选条件，如 {'start_time': datetime, 'end_time': datetime}
        
        返回格式: 每个优先级队列的每个消费者组作为一条记录
        """
        try:
            # 1. 从Redis获取队列和消费者组结构（只获取基本信息，不获取队列长度）
            all_stats = await self._get_queue_structure_from_redis()
            
            if not all_stats:
                return []
            
            # 2. 从数据库批量补充统计信息（队列长度、成功率等）
            await self._enrich_with_db_stats(all_stats, time_filter)
            
            # 3. 按基础队列名和优先级排序
            all_stats.sort(key=lambda x: (
                x['base_queue_name'],
                x['priority'] if x['priority'] is not None else 999,
                x['group_name'] or ''
            ))
            
            return all_stats
            
        except Exception as e:
            logger.error(f"获取队列统计信息失败: {e}")
            raise
    
    async def _get_queue_structure_from_redis(self) -> List[dict]:
        """从Redis获取队列和消费者组的基本结构，不获取队列长度"""
        try:
            # 1. 直接使用KEYS命令获取所有队列键（队列数量不多时性能更好）
            queue_pattern = f"{self.redis_prefix}:QUEUE:*"
            queue_keys = await self.redis_client.keys(queue_pattern)
            if not queue_keys:
                return []
            
            # 2. 批量获取消费者组信息（不获取队列长度）
            pipe = self.redis_client.pipeline()
            for queue_key in queue_keys:
                pipe.xinfo_groups(queue_key)
            
            try:
                groups_results = await pipe.execute()
            except redis.ResponseError:
                # 某些队列可能不是stream类型，逐个处理
                groups_results = []
                for queue_key in queue_keys:
                    try:
                        result = await self.redis_client.xinfo_groups(queue_key)
                        groups_results.append(result)
                    except redis.ResponseError:
                        groups_results.append([])
            # 3. 构建统计记录
            all_stats = []
            for i, queue_key in enumerate(queue_keys):
                # 处理bytes和str
                if isinstance(queue_key, bytes):
                    queue_key_str = queue_key.decode('utf-8')
                else:
                    queue_key_str = queue_key
                
                # 提取队列名
                queue_name = queue_key_str.replace(f"{self.redis_prefix}:QUEUE:", "")
                
                # 提取基础队列名和优先级
                base_queue_name = self._get_base_queue_name(queue_name)
                priority = self._extract_priority(queue_name)
                
                # 获取消费者组信息
                groups_info = groups_results[i] if i < len(groups_results) else []
                
                if groups_info:
                    for group in groups_info:
                        # 处理可能的bytes类型
                        group_name = group.get('name', group.get(b'name', ''))
                        if isinstance(group_name, bytes):
                            group_name = group_name.decode('utf-8')
                        
                        # 过滤掉内部消费者组
                        if is_internal_consumer(group_name):
                            continue
                        
                        consumers_count = group.get('consumers', group.get(b'consumers', 0))
                        pending_count = group.get('pending', group.get(b'pending', 0))
                        
                        last_delivered_id = group.get('last-delivered-id', group.get(b'last-delivered-id', ''))
                        if isinstance(last_delivered_id, bytes):
                            last_delivered_id = last_delivered_id.decode('utf-8')
                        
                        # 过滤掉从未处理过消息的消费者组（last_delivered_id='0-0'）
                        if last_delivered_id == '0-0':
                            continue
                        
                        # 提取任务名
                        task_name = self._extract_task_name(group_name)
                        
                        # 创建扁平化的记录
                        stat_record = {
                            'base_queue_name': base_queue_name,
                            'full_queue_name': queue_name,
                            'priority': priority,
                            'queue_length': 0,  # 将从数据库获取
                            'group_name': group_name,
                            'task_name': task_name,
                            'consumers': consumers_count,
                            'pending': pending_count,
                            'last_delivered_id': last_delivered_id,
                            'visible_messages': 0,  # 将计算
                            'invisible_messages': pending_count,
                            'success_count': 0,  # 将从数据库获取
                            'failed_count': 0,  # 将从数据库获取
                            'success_rate': 0.0,
                            'processing_rate': 0.0,
                            'avg_execution_time': 0.0
                        }
                        
                        all_stats.append(stat_record)
                else:
                    # 队列没有消费者组，仍然记录队列信息
                    stat_record = {
                        'base_queue_name': base_queue_name,
                        'full_queue_name': queue_name,
                        'priority': priority,
                        'queue_length': 0,  # 将从数据库获取
                        'group_name': None,
                        'task_name': None,
                        'consumers': 0,
                        'pending': 0,
                        'last_delivered_id': None,
                        'visible_messages': 0,
                        'invisible_messages': 0,
                        'success_count': 0,
                        'failed_count': 0,
                        'success_rate': 0.0,
                        'processing_rate': 0.0,
                        'avg_execution_time': 0.0
                    }
                    all_stats.append(stat_record)
            
            return all_stats
            
        except Exception as e:
            logger.error(f"从Redis获取队列结构失败: {e}")
            import traceback
            logger.error(traceback.format_exc())
            return []
    
    async def _enrich_with_db_stats(self, stats: List[dict], time_filter: Optional[Dict[str, Any]] = None):
        """从数据库批量补充统计信息"""
        if not self.pg_session or not stats:
            return
        
        try:
            # 收集所有唯一的队列和消费者组
            unique_queues = list(set(stat['full_queue_name'] for stat in stats))
            unique_groups = list(set(stat['group_name'] for stat in stats if stat['group_name']))
            
            if not unique_queues:
                return
            
            # 构建时间筛选条件
            time_clause = ""
            params = {
                'namespace': self.redis_prefix,
                'queues': unique_queues
            }
            
            if unique_groups:
                params['groups'] = unique_groups
            
            if time_filter:
                if 'start_time' in time_filter:
                    time_clause += " AND t.created_at >= :start_time"
                    params['start_time'] = time_filter['start_time']
                if 'end_time' in time_filter:
                    time_clause += " AND t.created_at <= :end_time"
                    params['end_time'] = time_filter['end_time']
            
            # 修复的查询 - 正确计算每个队列和消费者组的统计
            query = text(f"""
                WITH base_data AS (
                    -- 获取所有任务及其运行状态
                    SELECT 
                        t.queue,
                        t.stream_id,
                        tr.consumer_group,
                        tr.status,
                        tr.duration,
                        tr.end_time,
                        CASE WHEN tr.stream_id IS NULL THEN 1 ELSE 0 END as is_unprocessed
                    FROM tasks t
                    LEFT JOIN task_runs tr ON t.stream_id = tr.stream_id
                    WHERE t.namespace = :namespace 
                        AND t.queue = ANY(:queues) {time_clause}
                ),
                group_stats AS (
                    -- 按队列和消费者组分组统计
                    SELECT 
                        queue,
                        consumer_group,
                        COUNT(DISTINCT stream_id) as total_tasks,
                        SUM(is_unprocessed) as unprocessed_count,
                        COUNT(DISTINCT CASE WHEN status IS NOT NULL THEN stream_id END) as processed_tasks,
                        COUNT(DISTINCT CASE WHEN status = 'pending' THEN stream_id END) as pending_in_runs,
                        COUNT(DISTINCT CASE WHEN status = 'success' THEN stream_id END) as success_count,
                        COUNT(DISTINCT CASE WHEN status = 'error' THEN stream_id END) as error_count,
                        AVG(CASE WHEN status = 'success' AND duration IS NOT NULL THEN duration END) as avg_execution_time,
                        COUNT(DISTINCT CASE WHEN status = 'success' AND end_time >= NOW() - INTERVAL '1 minute' THEN stream_id END) as recent_completed
                    FROM base_data
                    WHERE consumer_group IS NOT NULL
                    GROUP BY queue, consumer_group
                ),
                queue_totals AS (
                    -- 获取每个队列的总体统计（包括未处理的任务）
                    SELECT 
                        queue,
                        COUNT(DISTINCT stream_id) as queue_total_tasks,
                        SUM(is_unprocessed) as queue_unprocessed_tasks
                    FROM base_data
                    GROUP BY queue
                )
                -- 合并结果
                SELECT 
                    gs.queue,
                    gs.consumer_group,
                    COALESCE(qt.queue_total_tasks, gs.total_tasks) as total_tasks,
                    gs.processed_tasks,
                    COALESCE(qt.queue_unprocessed_tasks, 0) as unprocessed_tasks,
                    gs.pending_in_runs,
                    gs.success_count,
                    gs.error_count,
                    gs.avg_execution_time,
                    gs.recent_completed
                FROM group_stats gs
                LEFT JOIN queue_totals qt ON gs.queue = qt.queue
                
                UNION ALL
                
                -- 对于没有消费者组的队列，返回未处理任务的统计
                SELECT 
                    queue,
                    NULL as consumer_group,
                    queue_total_tasks as total_tasks,
                    0 as processed_tasks,
                    queue_unprocessed_tasks as unprocessed_tasks,
                    0 as pending_in_runs,
                    0 as success_count,
                    0 as error_count,
                    NULL as avg_execution_time,
                    0 as recent_completed
                FROM queue_totals
                WHERE queue_unprocessed_tasks > 0
            """)
            
            result = await self.pg_session.execute(query, params)
            # 构建结果映射
            db_stats = {}
            for row in result:
                if row.consumer_group:
                    key = f"{row.queue}|{row.consumer_group}"
                else:
                    key = f"{row.queue}|NONE"
                db_stats[key] = row
            
            # 更新stats数组
            for stat in stats:
                if stat['group_name']:
                    key = f"{stat['full_queue_name']}|{stat['group_name']}"
                else:
                    key = f"{stat['full_queue_name']}|NONE"
                
                if key in db_stats:
                    row = db_stats[key]
                    stat['queue_length'] = row.total_tasks or 0
                    stat['success_count'] = row.success_count or 0
                    stat['failed_count'] = row.error_count or 0
                    stat['avg_execution_time'] = float(row.avg_execution_time or 0)
                    stat['processing_rate'] = row.recent_completed or 0
                    
                    # 计算成功率
                    total = stat['success_count'] + stat['failed_count']
                    if total > 0:
                        stat['success_rate'] = round((stat['success_count'] / total) * 100, 2)
                    
                    # 改进的可见消息计算逻辑
                    # 可见消息 = 未被处理的任务（tasks有但task_runs没有） + (task_runs中pending的 - Redis中pending的)
                    unprocessed_tasks = row.unprocessed_tasks or 0  # tasks表有但task_runs表没有的
                    pending_in_runs = row.pending_in_runs or 0      # task_runs表中status='pending'的
                    redis_pending = stat['pending']                  # Redis中pending的
                    
                    # 未被worker领取的 + 已完成但未确认的
                    stat['visible_messages'] = unprocessed_tasks + max(0, pending_in_runs - redis_pending)
                    
        except Exception as e:
            logger.error(f"从数据库补充统计信息失败: {e}")
            import traceback
            logger.error(traceback.format_exc())
    
    
    async def _get_active_workers_for_queue(self, base_queue_name: str) -> int:
        """获取指定基础队列的活跃Worker数量"""
        try:
            import time
            active_workers = 0
            
            # 扫描所有worker键
            worker_pattern = f"{self.redis_prefix}:WORKER:*"
            worker_keys = await self.redis_client.keys(worker_pattern)
            
            for worker_key in worker_keys:
                try:
                    worker_info = await self.redis_client.hgetall(worker_key)
                    if worker_info:
                        # 检查心跳是否在60秒内
                        last_heartbeat = worker_info.get(b'last_heartbeat', worker_info.get('last_heartbeat'))
                        if last_heartbeat:
                            if isinstance(last_heartbeat, bytes):
                                last_heartbeat = last_heartbeat.decode('utf-8')
                            heartbeat_time = float(last_heartbeat)
                            if time.time() - heartbeat_time < 60:  # 60秒内有心跳
                                # 检查队列列表是否包含基础队列名
                                queues = worker_info.get(b'queues', worker_info.get('queues', ''))
                                if isinstance(queues, bytes):
                                    queues = queues.decode('utf-8')
                                if base_queue_name in queues:
                                    active_workers += 1
                except Exception:
                    continue
                    
            return active_workers
        except Exception as e:
            logger.warning(f"获取活跃workers失败: {e}")
            return 0
    
    async def _transform_to_grouped_format(self, flat_stats: List[dict]) -> List[dict]:
        """将扁平化的统计数据转换为按队列分组的嵌套格式"""
        grouped_data = {}
        
        for stat in flat_stats:
            base_queue_name = stat['base_queue_name']
            
            if base_queue_name not in grouped_data:
                grouped_data[base_queue_name] = {
                    'queue_name': base_queue_name,
                    'consumer_groups': [],
                    'consumer_groups_count': 0,
                    'total_length': 0,
                    'priority_queues': set(),  # 用于收集所有优先级队列
                    'total_visible': 0,
                    'total_invisible': 0,
                    'total_success': 0,
                    'total_failed': 0,
                    'active_workers': 0  # 添加活跃Workers字段
                }
            
            # 添加消费者组信息
            if stat['group_name']:
                consumer_group = {
                    'group_name': stat['group_name'],
                    'task_name': stat['task_name'],
                    'queue_name': stat['full_queue_name'],  # 包含优先级的完整队列名
                    'priority': stat['priority'],
                    'queue_length': stat['queue_length'],
                    'consumers': stat['consumers'],
                    'pending': stat['pending'],
                    'last_delivered_id': stat['last_delivered_id'],
                    'visible_messages': stat['visible_messages'],
                    'invisible_messages': stat['invisible_messages'],
                    'success_count': stat['success_count'],
                    'failed_count': stat['failed_count'],
                    'success_rate': stat['success_rate'],
                    'processing_rate': stat['processing_rate'],
                    'avg_execution_time': stat['avg_execution_time'],
                    'unique_key': f"{stat['group_name']}_{stat['full_queue_name']}"  # 唯一键
                }
                grouped_data[base_queue_name]['consumer_groups'].append(consumer_group)
                
                # 累加统计
                grouped_data[base_queue_name]['total_visible'] += stat['visible_messages']
                grouped_data[base_queue_name]['total_invisible'] += stat['invisible_messages']
                grouped_data[base_queue_name]['total_success'] += stat['success_count']
                grouped_data[base_queue_name]['total_failed'] += stat['failed_count']
            
            # 收集优先级队列信息（现在priority不会是None，0表示无优先级）
            grouped_data[base_queue_name]['priority_queues'].add(stat['priority'])
            
            # 更新队列长度（累加所有优先级队列的长度）
            # 使用字典记录已计算的队列，避免重复
            if base_queue_name not in grouped_data:
                grouped_data[base_queue_name]['calculated_queues'] = set()
            
            full_queue = stat['full_queue_name']
            if full_queue not in grouped_data[base_queue_name].get('calculated_queues', set()):
                grouped_data[base_queue_name]['total_length'] += stat['queue_length']
                grouped_data[base_queue_name].setdefault('calculated_queues', set()).add(full_queue)
        
        # 转换为列表并处理优先级队列
        result = []
        for queue_data in grouped_data.values():
            # 清理临时字段
            queue_data.pop('calculated_queues', None)
            
            # 转换优先级队列集合为排序列表
            queue_data['priority_queues'] = sorted(list(queue_data['priority_queues']))
            queue_data['consumer_groups_count'] = len(queue_data['consumer_groups'])
            
            # 获取该基础队列的活跃Workers数
            queue_data['active_workers'] = await self._get_active_workers_for_queue(queue_data['queue_name'])
            
            # 计算总体成功率
            total_tasks = queue_data['total_success'] + queue_data['total_failed']
            if total_tasks > 0:
                queue_data['overall_success_rate'] = round(
                    (queue_data['total_success'] / total_tasks) * 100, 2
                )
            else:
                queue_data['overall_success_rate'] = 0.0
            
            result.append(queue_data)
        
        # 按队列名排序
        result.sort(key=lambda x: x['queue_name'])
        
        return result
    
    
    def _get_base_queue_name(self, queue_name: str) -> str:
        """提取基础队列名（去除优先级后缀）"""
        # 检查是否包含优先级后缀（格式: queue_name:priority）
        if ':' in queue_name:
            parts = queue_name.rsplit(':', 1)
            # 检查最后一部分是否是数字（优先级）
            if parts[-1].isdigit():
                return parts[0]
        return queue_name
    
    def _extract_priority(self, queue_name: str) -> int:
        """从队列名中提取优先级，无优先级后缀的返回0"""
        if ':' in queue_name:
            parts = queue_name.rsplit(':', 1)
            if parts[-1].isdigit():
                return int(parts[-1])
        return 0  # 没有优先级后缀的队列默认优先级为0
    
    def _extract_task_name(self, group_name: str) -> str:
        """从消费者组名中提取任务名"""
        # 消费者组名格式: prefix:QUEUE:queue_name:task_name
        # 或: prefix:QUEUE:queue_name（默认组）
        if ':' in group_name:
            parts = group_name.split(':')
            # 查找QUEUE后的部分
            try:
                queue_idx = parts.index('QUEUE')
                # task_name是QUEUE后的第二个部分（如果存在）
                if len(parts) > queue_idx + 2:
                    # 返回最后一个部分作为task_name
                    return parts[-1]
            except ValueError:
                pass
        return 'default'
    
    def _is_default_idle_group(self, group_name: str, base_queue_name: str) -> bool:
        """检查是否是空闲的默认消费者组"""
        # 默认组的格式是: prefix:QUEUE:queue_name（没有task后缀）
        if ':QUEUE:' in group_name:
            parts = group_name.split(':QUEUE:')
            if len(parts) == 2:
                queue_part = parts[1]
                # 如果queue_part就是base_queue_name或带优先级版本，说明是默认组
                if queue_part == base_queue_name or queue_part.startswith(f"{base_queue_name}:"):
                    # 检查是否没有task后缀
                    if ':' not in queue_part.replace(f"{base_queue_name}:", ""):
                        return True
        return False


"""任务持久化模块

负责解析Redis Stream消息，并将任务数据批量插入PostgreSQL数据库。
"""

import json
import logging
import traceback
from typing import Dict, List, Optional, Any
from datetime import datetime, timezone

from sqlalchemy.orm import sessionmaker
from sqlalchemy.dialects.postgresql import insert

from jettask.db.models.task import Task

logger = logging.getLogger(__name__)


class TaskPersistence:
    """任务持久化处理器

    职责：
    - 解析Stream消息为任务信息
    - 批量插入任务到PostgreSQL的tasks表
    - 处理插入失败的降级策略
    """

    def __init__(
        self,
        async_session_local: sessionmaker,
        namespace_id: str,
        namespace_name: str
    ):
        """初始化任务持久化处理器

        Args:
            async_session_local: SQLAlchemy会话工厂
            namespace_id: 命名空间ID
            namespace_name: 命名空间名称
        """
        self.AsyncSessionLocal = async_session_local
        self.namespace_id = namespace_id
        self.namespace_name = namespace_name

    def parse_stream_message(self, task_id: str, data: dict) -> Optional[dict]:
        """解析Stream消息为任务信息（返回完整的字段）

        Args:
            task_id: 任务ID（Redis Stream ID）
            data: 消息数据

        Returns:
            解析后的任务信息字典，失败返回None
        """
        try:
            from jettask.utils.serializer import loads_str

            if b'data' in data:
                task_data = loads_str(data[b'data'])
            else:
                task_data = {}
                for k, v in data.items():
                    key = k.decode('utf-8') if isinstance(k, bytes) else k
                    if isinstance(v, bytes):
                        try:
                            value = loads_str(v)
                        except:
                            value = str(v)
                    else:
                        value = v
                    task_data[key] = value

            # 如果配置了命名空间，检查消息是否属于该命名空间
            # if self.namespace_id:
            #     msg_namespace_id = task_data.get('__namespace_id')
            #     # 如果消息没有namespace_id且当前不是默认命名空间，跳过
            #     if msg_namespace_id != self.namespace_id:
            #         if not (msg_namespace_id is None and self.namespace_id == 'default'):
            #             logger.debug(f"Skipping message from different namespace: {msg_namespace_id} != {self.namespace_id}")
            #             return None

            queue_name = task_data['queue']
            task_name = task_data.get('name', task_data.get('task', 'unknown'))

            created_at = None
            if 'trigger_time' in task_data:
                try:
                    timestamp = float(task_data['trigger_time'])
                    created_at = datetime.fromtimestamp(timestamp, tz=timezone.utc)
                except:
                    pass

            # 返回完整的字段，包括所有可能为None的字段
            return {
                'id': task_id,
                'queue_name': queue_name,
                'task_name': task_name,
                'task_data': json.dumps(task_data),
                'priority': int(task_data.get('priority', 0)),
                'delay': float(task_data.get('delay', 0)),  # 🔧 添加 delay 字段
                'retry_count': int(task_data.get('retry', 0)),
                'max_retry': int(task_data.get('max_retry', 3)),
                'status': 'pending',
                'result': None,  # 新任务没有结果
                'error_message': None,  # 新任务没有错误信息
                'created_at': created_at,
                'started_at': None,  # 新任务还未开始
                'completed_at': None,  # 新任务还未完成
                'scheduled_task_id': task_data.get('scheduled_task_id'),  # 调度任务ID
                'metadata': json.dumps(task_data.get('metadata', {})),
                'worker_id': None,  # 新任务还未分配worker
                'execution_time': None,  # 新任务还没有执行时间
                'duration': None,  # 新任务还没有持续时间
                'namespace_id': self.namespace_id  # 添加命名空间ID
            }

        except Exception as e:
            logger.error(f"Error parsing stream message for task {task_id}: {e}")
            logger.error(traceback.format_exc())
            return None

    async def insert_tasks(self, tasks: List[Dict[str, Any]]) -> int:
        """批量插入任务到PostgreSQL（使用ORM）

        Args:
            tasks: 任务信息列表

        Returns:
            实际插入的记录数
        """
        if not tasks:
            return 0

        logger.info(f"Attempting to insert {len(tasks)} tasks to tasks table")

        try:
            async with self.AsyncSessionLocal() as session:
                # 准备tasks表的数据
                tasks_data = []
                for task in tasks:
                    task_data = json.loads(task['task_data'])

                    # 从task_data中获取scheduled_task_id
                    scheduled_task_id = task_data.get('scheduled_task_id') or task.get('scheduled_task_id')

                    # 根据是否有scheduled_task_id来判断任务来源
                    if scheduled_task_id:
                        source = 'scheduler'  # 定时任务
                    else:
                        source = 'redis_stream'  # 普通任务

                    tasks_data.append({
                        'stream_id': task['id'],  # Redis Stream ID作为stream_id
                        'queue': task['queue_name'],
                        'namespace': self.namespace_name,
                        'scheduled_task_id': str(scheduled_task_id) if scheduled_task_id else None,
                        'payload': json.loads(task['task_data']),  # 解析为dict
                        'priority': task['priority'],
                        'delay': task.get('delay'),  # 🔧 添加 delay
                        'created_at': task['created_at'],
                        'source': source,
                        'task_metadata': json.loads(task.get('metadata', '{}'))  # 对应模型的 task_metadata 字段
                    })

                # 批量插入 - 使用 ORM 的 INSERT ON CONFLICT DO NOTHING
                logger.debug(f"Executing batch insert with {len(tasks_data)} tasks")

                try:
                    # 使用 PostgreSQL 的 insert().on_conflict_do_nothing()
                    stmt = insert(Task).values(tasks_data).on_conflict_do_nothing(
                        constraint='tasks_pkey'  # 主键冲突则跳过
                    )

                    await session.execute(stmt)
                    await session.commit()

                    # ORM 的 on_conflict_do_nothing 不返回 rowcount，我们假设全部成功
                    inserted_count = len(tasks_data)
                    logger.debug(f"Tasks table batch insert transaction completed: {inserted_count} tasks")
                    return inserted_count

                except Exception as e:
                    logger.error(f"Error in batch insert, trying fallback: {e}")
                    await session.rollback()

                    # 降级为逐条插入（更稳妥）
                    total_inserted = 0

                    for task_dict in tasks_data:
                        try:
                            stmt = insert(Task).values(**task_dict).on_conflict_do_nothing(
                                constraint='tasks_pkey'
                            )
                            await session.execute(stmt)
                            await session.commit()
                            total_inserted += 1
                        except Exception as single_error:
                            logger.error(f"Failed to insert task {task_dict.get('stream_id')}: {single_error}")
                            await session.rollback()

                    if total_inserted > 0:
                        logger.info(f"Fallback insert completed: {total_inserted} tasks inserted")
                    else:
                        logger.info(f"No new tasks inserted in fallback mode")

                    return total_inserted

        except Exception as e:
            logger.error(f"Error inserting tasks to PostgreSQL: {e}")
            logger.error(traceback.format_exc())
            return 0

    async def batch_insert_tasks(self, tasks: List[Dict[str, Any]]) -> int:
        """批量插入任务（兼容 buffer.py 调用接口）

        Args:
            tasks: 任务记录列表

        Returns:
            实际插入的记录数
        """
        if not tasks:
            return 0

        logger.info(f"[BATCH INSERT] 批量插入 {len(tasks)} 条任务...")

        try:
            async with self.AsyncSessionLocal() as session:
                # 准备 ORM 数据
                insert_data = []
                for record in tasks:
                    print(f'{record=}')
                    # record 是从 consumer.py 传入的格式
                    insert_data.append({
                        'stream_id': record['stream_id'],
                        'queue': record['queue'],
                        'namespace': record['namespace'],
                        'scheduled_task_id': record.get('scheduled_task_id'),
                        'payload': record.get('payload', {}),
                        'priority': record.get('priority', 0),
                        'delay': record.get('delay', 0),
                        'created_at': record.get('created_at'),
                        'source': record.get('source', 'redis_stream'),
                        'task_metadata': record.get('metadata', {})
                    })

                # 批量插入 - 使用 PostgreSQL 的 INSERT ON CONFLICT DO NOTHING
                # 使用约束名称而不是列名
                stmt = insert(Task).values(insert_data).on_conflict_do_nothing(
                    constraint='tasks_pkey'
                )

                await session.execute(stmt)
                await session.commit()

                logger.info(f"[BATCH INSERT] ✓ 成功插入 {len(insert_data)} 条任务")
                return len(insert_data)

        except Exception as e:
            logger.error(f"[BATCH INSERT] ✗ 批量插入失败: {e}", exc_info=True)
            return 0

    async def batch_update_tasks(self, updates: List[Dict[str, Any]]) -> int:
        """批量更新任务执行状态到 task_runs 表

        使用 PostgreSQL 的 INSERT ... ON CONFLICT DO UPDATE 实现 UPSERT 操作，
        如果记录存在则更新，不存在则插入。

        Args:
            updates: 更新记录列表，每条记录包含：
                - stream_id: Redis Stream ID（主键）
                - status: 任务状态
                - result: 执行结果
                - error: 错误信息
                - started_at: 开始时间
                - completed_at: 完成时间
                - retries: 重试次数

        Returns:
            实际更新的记录数
        """
        if not updates:
            return 0

        # logger.info(f"[BATCH UPDATE] 批量更新 {len(updates)} 条任务状态...")
        # logger.info(f"[BATCH UPDATE] 更新记录示例: {updates[0] if updates else 'N/A'}")

        try:
            from sqlalchemy.dialects.postgresql import insert
            from ..db.models import TaskRun
            from ..utils.serializer import loads_str
            from datetime import datetime, timezone

            # 对相同 stream_id 的记录进行去重，保留最新的
            # 使用字典，key 是 stream_id，value 是记录（后面的会覆盖前面的）
            deduplicated = {}
            for record in updates:
                stream_id = record['stream_id']
                deduplicated[stream_id] = record

            # 转换回列表
            unique_updates = list(deduplicated.values())

            if len(unique_updates) < len(updates):
                logger.info(
                    f"[BATCH UPDATE] 去重: {len(updates)} 条 → {len(unique_updates)} 条 "
                    f"(合并了 {len(updates) - len(unique_updates)} 条重复记录)"
                )

            async with self.AsyncSessionLocal() as session:
                # 准备 UPSERT 数据
                upsert_data = []
                for record in unique_updates:
                    logger.debug(f"处理记录: {record}")
                    # 解析 result 字段（如果是序列化的字符串）
                    result = record.get('result')
                    if result and isinstance(result, bytes):
                        try:
                            result = loads_str(result)
                        except Exception:
                            result = result.decode('utf-8') if isinstance(result, bytes) else result

                    # 解析 error 字段
                    error = record.get('error')
                    if error and isinstance(error, bytes):
                        error = error.decode('utf-8')

                    # 计算执行时长
                    duration = None
                    started_at = record.get('started_at')
                    completed_at = record.get('completed_at')
                    if started_at and completed_at:
                        duration = completed_at - started_at

                    # 解析 status 字段
                    status = record.get('status')
                    if status and isinstance(status, bytes):
                        status = status.decode('utf-8')

                    # 解析 consumer 字段
                    consumer = record.get('consumer')
                    if consumer and isinstance(consumer, bytes):
                        consumer = consumer.decode('utf-8')

                    # 🔧 获取 task_name（已从 consumer 提取）
                    task_name = record.get('task_name')

                    upsert_record = {
                        'stream_id': record['stream_id'],
                        'task_name': task_name,  # 🔧 添加 task_name
                        'status': status,
                        'result': result,
                        'error': error,
                        'started_at': started_at,
                        'completed_at': completed_at,
                        'retries': record.get('retries', 0),
                        'duration': duration,
                        'consumer': consumer,
                        'updated_at': datetime.now(timezone.utc),
                    }
                    logger.debug(f"upsert_record: {upsert_record}")
                    upsert_data.append(upsert_record)

                logger.info(f"[BATCH UPDATE] 准备写入 {len(upsert_data)} 条记录")

                # 批量 UPSERT - 如果存在则更新，不存在则插入
                stmt = insert(TaskRun).values(upsert_data)

                # 定义冲突时的更新策略
                # 使用 COALESCE 避免用 NULL 覆盖已有数据
                from sqlalchemy import func
                stmt = stmt.on_conflict_do_update(
                    constraint='task_runs_pkey',
                    set_={
                        # status 总是更新（状态变化）
                        'status': stmt.excluded.status,
                        # 其他字段：如果新值不是 NULL，则更新；否则保留旧值
                        'result': func.coalesce(stmt.excluded.result, TaskRun.result),
                        'error': func.coalesce(stmt.excluded.error, TaskRun.error),
                        'started_at': func.coalesce(stmt.excluded.started_at, TaskRun.started_at),
                        'completed_at': func.coalesce(stmt.excluded.completed_at, TaskRun.completed_at),
                        'retries': func.coalesce(stmt.excluded.retries, TaskRun.retries),
                        'duration': func.coalesce(stmt.excluded.duration, TaskRun.duration),
                        'consumer': func.coalesce(stmt.excluded.consumer, TaskRun.consumer),
                        # updated_at 总是更新
                        'updated_at': stmt.excluded.updated_at,
                    }
                )

                await session.execute(stmt)
                await session.commit()

                logger.info(f"[BATCH UPDATE] ✓ 成功更新 {len(upsert_data)} 条任务状态")
                return len(upsert_data)

        except Exception as e:
            logger.error(f"[BATCH UPDATE] ✗ 批量更新失败: {e}", exc_info=True)
            return 0



"""
Redis Lua 脚本定义

集中管理所有的 Redis Lua 脚本，用于原子操作和性能优化
"""

# Lua脚本：原子获取并删除结果
# 用于获取任务结果后立即删除，避免多次读取
LUA_SCRIPT_GET_AND_DELETE = """
local key = KEYS[1]
local value = redis.call('GET', key)
if value then
    redis.call('DEL', key)
end
return value
"""

# Lua脚本：批量发送任务（简化版，已被LUA_SCRIPT_NORMAL_TASKS替代）
# 保留用于兼容性
LUA_SCRIPT_BATCH_SEND = """
local stream_key = KEYS[1]
local task_name = ARGV[1]
local count = 0

for i = 2, #ARGV do
    redis.call('XADD', stream_key, '*', 'task_name', task_name, 'message', ARGV[i])
    count = count + 1
end

return count
"""


# Lua脚本：批量发送事件
# 用于 _batch_send_event 和 _batch_send_event_sync 方法
# 批量发送消息到Stream并自动生成offset，同时注册队列
LUA_SCRIPT_BATCH_SEND_EVENT = """
local stream_key = KEYS[1]
local prefix = ARGV[1]
local results = {}

-- 使用Hash存储所有队列的offset
local offsets_hash = prefix .. ':QUEUE_OFFSETS'

-- 从stream_key中提取队列名（去掉prefix:QUEUE:前缀）
local queue_name = string.gsub(stream_key, '^' .. prefix .. ':QUEUE:', '')

-- 将队列添加到全局队列注册表（包括所有队列，包括优先级队列）
local queues_registry_key = prefix .. ':REGISTRY:QUEUES'
redis.call('SADD', queues_registry_key, queue_name)

-- 从ARGV[2]开始，每个参数是一个消息的data
for i = 2, #ARGV do
    local data = ARGV[i]

    -- 使用HINCRBY原子递增offset（如果不存在会自动创建并设为1）
    local current_offset = redis.call('HINCRBY', offsets_hash, queue_name, 1)

    -- 添加消息到Stream（包含offset字段）
    local stream_id = redis.call('XADD', stream_key, '*',
        'data', data,
        'offset', current_offset)

    table.insert(results, stream_id)
end

return results
"""

__all__ = [
    'LUA_SCRIPT_GET_AND_DELETE',
    'LUA_SCRIPT_BATCH_SEND',
    'LUA_SCRIPT_BATCH_SEND_EVENT',
]



"""
队列监控服务

提供队列相关的监控功能
"""
import logging
import time
import json
from typing import Dict, Any, List, Optional
from datetime import datetime

from .redis_monitor_service import RedisMonitorService

logger = logging.getLogger(__name__)


class QueueMonitorService:
    """队列监控服务类"""

    def __init__(self, redis_service: RedisMonitorService):
        """
        初始化队列监控服务

        Args:
            redis_service: Redis 监控基础服务实例
        """
        self.redis_service = redis_service

    @property
    def redis(self):
        """获取 Redis 客户端"""
        return self.redis_service.redis

    @property
    def redis_prefix(self) -> str:
        """获取 Redis 前缀"""
        return self.redis_service.redis_prefix

    async def get_all_queues(self) -> List[str]:
        """
        获取所有队列名称

        Returns:
            队列名称列表（已排序）
        """
        try:
            # 检查缓存是否有效
            current_time = time.time()
            if (self.redis_service._queues_cache is not None and
                    (current_time - self.redis_service._queues_cache_time) < self.redis_service._queues_cache_ttl):
                logger.debug("Returning cached queue list")
                return self.redis_service._queues_cache

            # 优先尝试从全局队列集合获取
            global_queues_key = f'{self.redis_prefix}:global:queues'
            queues = await self.redis.smembers(global_queues_key)

            if queues:
                # 如果有全局队列集合，直接使用
                result = sorted(list(queues))
                self.redis_service._queues_cache = result
                self.redis_service._queues_cache_time = current_time
                logger.info(f"Retrieved {len(result)} queues from global set")
                return result

            # 如果没有全局队列集合，使用 QueueRegistry
            from jettask.messaging.registry import QueueRegistry

            queue_registry = QueueRegistry(
                redis_client=None,
                async_redis_client=self.redis,
                redis_prefix=self.redis_prefix
            )

            # 获取所有队列
            queues = await queue_registry.get_all_queues()

            # 返回排序后的队列列表并更新缓存
            result = sorted(list(queues))
            self.redis_service._queues_cache = result
            self.redis_service._queues_cache_time = current_time
            logger.info(f"Retrieved {len(result)} queues from registry")
            return result

        except Exception as e:
            logger.error(f"Error getting all queues: {e}", exc_info=True)
            return []

    async def get_queue_stats(self, queue_name: str) -> Dict[str, Any]:
        """
        获取队列统计信息（RabbitMQ 兼容格式）

        Args:
            queue_name: 队列名称

        Returns:
            队列统计信息字典
        """
        prefixed_queue_name = self.redis_service.get_prefixed_queue_name(queue_name)

        try:
            # 获取 Stream 信息和消费者组信息
            info = await self.redis.xinfo_stream(prefixed_queue_name)
            groups = await self.redis.xinfo_groups(prefixed_queue_name)

        except Exception as e:
            # 如果队列不存在，返回默认值
            logger.warning(f"Queue {queue_name} does not exist or has no groups: {e}")
            return {
                "queue": queue_name,
                "messages": 0,
                "messages_ready": 0,
                "messages_unacknowledged": 0,
                "consumers": 0,
                "message_stats": {
                    "publish": 0,
                    "deliver_get": 0,
                    "ack": 0
                },
                "consumer_groups": [],
                "error": str(e)
            }

        # 计算基础指标
        total_messages = info["length"]
        total_pending = 0
        total_consumers = 0
        total_delivered = 0

        consumer_groups_info = []

        for group in groups:
            group_pending = group["pending"]
            group_consumers_count = group["consumers"]

            total_pending += group_pending
            total_consumers += group_consumers_count

            group_info = {
                "name": group["name"],
                "consumers": group_consumers_count,
                "pending": group_pending,
                "last_delivered_id": group["last-delivered-id"]
            }

            # 获取消费者详情
            try:
                consumers = await self.redis.xinfo_consumers(prefixed_queue_name, group["name"])
                group_info["consumer_details"] = consumers

                # 从消费者统计中计算 deliver 数量
                for consumer in consumers:
                    total_delivered += consumer.get("pel-count", 0)

            except Exception as e:
                group_info["consumer_details"] = []
                logger.warning(f"Error getting consumers for group {group['name']}: {e}")

            consumer_groups_info.append(group_info)

        # 从 Worker 统计中获取更精确的消息统计（需要 Worker 监控服务）
        try:
            from .worker_monitor_service import WorkerMonitorService
            worker_service = WorkerMonitorService(self.redis_service)
            worker_summary = await worker_service.get_queue_worker_summary_fast(queue_name)

            publish_count = worker_summary.get('total_count', 0)
            deliver_count = worker_summary.get('total_success_count', 0) + worker_summary.get('total_failed_count', 0)
            ack_count = worker_summary.get('total_success_count', 0)
            avg_processing_time = worker_summary.get('avg_processing_time', 0.0)
            avg_latency_time = worker_summary.get('avg_latency_time', 0.0)
            total_running_tasks = worker_summary.get('total_running_tasks', 0)
        except Exception as e:
            logger.warning(f"Error getting worker summary for queue {queue_name}: {e}")
            publish_count = 0
            deliver_count = 0
            ack_count = 0
            avg_processing_time = 0.0
            avg_latency_time = 0.0
            total_running_tasks = 0

        # 计算就绪消息数（队列总长度 - 未确认消息数）
        messages_ready = max(0, total_messages - total_pending)

        # RabbitMQ 风格的统计信息
        stats = {
            "queue": queue_name,
            # RabbitMQ 兼容指标
            "messages": total_messages,  # 队列中消息总数
            "messages_ready": messages_ready,  # 就绪状态的消息数
            "messages_unacknowledged": total_pending,  # 未确认的消息数
            "consumers": total_consumers,  # 消费者数量
            "message_stats": {
                "publish": publish_count,  # 发布到队列的消息数量
                "deliver_get": deliver_count,  # 被消费的消息数量
                "ack": ack_count  # 被确认的消息数量
            },
            # 原有详细信息保持兼容性
            "length": info["length"],
            "first_entry": info.get("first-entry"),
            "last_entry": info.get("last-entry"),
            "consumer_groups": consumer_groups_info,
            # 额外的性能指标
            "performance_stats": {
                "avg_processing_time": avg_processing_time,
                "avg_latency_time": avg_latency_time,
                "total_running_tasks": total_running_tasks
            }
        }

        logger.debug(f"Queue stats for {queue_name}: {total_messages} messages, {total_consumers} consumers")
        return stats

    async def get_stream_info(self, queue_name: str, event_id: str) -> Optional[Dict[str, Any]]:
        """
        从 Stream 中获取任务详细信息

        Args:
            queue_name: 队列名称
            event_id: 事件 ID 或消息 ID

        Returns:
            任务信息字典，如果未找到则返回 None
        """
        try:
            prefixed_queue_name = self.redis_service.get_prefixed_queue_name(queue_name)

            # 先尝试按 event_id 直接查找
            messages = await self.redis.xrange(prefixed_queue_name, min=event_id, max=event_id, count=1)

            if messages:
                msg_id, data = messages[0]
                logger.debug(f"Found task {event_id} in queue {queue_name}")
                return {
                    "message_id": msg_id,
                    "data": data,
                    "queue": queue_name
                }

            # 如果没找到，可能 event_id 是消息内容的一部分，扫描最近的消息
            messages = await self.redis.xrange(prefixed_queue_name, count=100)
            for msg_id, data in messages:
                if data.get("event_id") == event_id or data.get("id") == event_id:
                    logger.debug(f"Found task {event_id} in recent messages")
                    return {
                        "message_id": msg_id,
                        "data": data,
                        "queue": queue_name
                    }

            logger.warning(f"Task {event_id} not found in queue {queue_name}")
            return None

        except Exception as e:
            logger.error(f"Error reading from stream {prefixed_queue_name}: {e}", exc_info=True)
            return None

    async def get_queue_tasks(
        self,
        queue_name: str,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        limit: int = 100
    ) -> Dict[str, Any]:
        """
        获取指定队列的任务（基于时间范围）

        Args:
            queue_name: 队列名称
            start_time: 开始时间（Redis Stream ID 格式或时间戳）
            end_time: 结束时间（Redis Stream ID 格式或时间戳）
            limit: 返回的最大任务数

        Returns:
            包含任务列表和分页信息的字典
        """
        all_tasks = []

        try:
            # 处理时间参数
            if not end_time:
                end_time = '+'
            if not start_time:
                start_time = '-'

            # 从队列的 stream 中读取消息
            prefixed_queue_name = self.redis_service.get_prefixed_queue_name(queue_name)
            messages = await self.redis.xrevrange(
                prefixed_queue_name,
                max=end_time,
                min=start_time,
                count=limit
            )

            for msg_id, data in messages:
                event_id = msg_id

                # 构建任务信息
                task_info = {
                    "event_id": event_id,
                    "message_id": msg_id,
                    "stream_data": data,
                    "task_name": data.get("name", "unknown"),
                    "queue": data.get("queue", queue_name),
                    "trigger_time": data.get("trigger_time")
                }

                # 尝试解析 args 和 kwargs
                params_str = self._parse_task_params(data)
                task_info["params_str"] = params_str

                # 从状态键获取信息
                status_info = await self._get_task_status(event_id, queue_name, data)
                task_info.update(status_info)

                all_tasks.append(task_info)

            logger.info(f"Retrieved {len(all_tasks)} tasks from queue {queue_name}")

        except Exception as e:
            logger.error(f"Error reading queue {queue_name}: {e}", exc_info=True)
            return {
                "tasks": [],
                "count": 0,
                "oldest_id": None,
                "newest_id": None,
                "has_more": False,
                "limit": limit
            }

        # 获取最早和最晚的消息 ID 用于分页导航
        oldest_id = all_tasks[-1]["message_id"] if all_tasks else None
        newest_id = all_tasks[0]["message_id"] if all_tasks else None

        return {
            "tasks": all_tasks,
            "count": len(all_tasks),
            "oldest_id": oldest_id,
            "newest_id": newest_id,
            "has_more": len(all_tasks) >= limit,
            "limit": limit
        }

    def _parse_task_params(self, data: Dict[str, Any]) -> str:
        """
        解析任务参数

        Args:
            data: 任务数据

        Returns:
            参数字符串
        """
        try:
            args_list = []
            kwargs_dict = {}

            if data.get("args"):
                args_list = json.loads(data["args"])

            if data.get("kwargs"):
                kwargs_dict = json.loads(data["kwargs"])

            # 构建参数字符串
            params_parts = []
            if args_list:
                params_parts.extend([str(arg) for arg in args_list])
            if kwargs_dict:
                params_parts.extend([f"{k}={v}" for k, v in kwargs_dict.items()])

            return ", ".join(params_parts) if params_parts else "无参数"

        except Exception as e:
            logger.warning(f"Error parsing task params: {e}")
            return "解析失败"

    async def _get_task_status(
        self,
        event_id: str,
        queue_name: str,
        data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        获取任务状态

        Args:
            event_id: 事件 ID
            queue_name: 队列名称
            data: 任务数据

        Returns:
            包含状态信息的字典
        """
        status_key = f"{self.redis_prefix}:STATUS:{event_id}"
        status = await self.redis.get(status_key)

        if status:
            try:
                parsed_status = json.loads(status)
                return {
                    "status": status,
                    "parsed_status": parsed_status,
                    "consumer": parsed_status.get("consumer", "-")
                }
            except Exception as e:
                logger.warning(f"Error parsing status for task {event_id}: {e}")
                return {
                    "status": status,
                    "parsed_status": {"status": "unknown"},
                    "consumer": "-"
                }
        else:
            # 构建默认状态
            default_status = {
                "status": "未知",
                "queue": queue_name,
                "created_at": datetime.fromtimestamp(
                    float(data.get("trigger_time", 0))
                ).isoformat() if data.get("trigger_time") else None
            }

            return {
                "status": json.dumps(default_status),
                "parsed_status": default_status,
                "consumer": "-"
            }



"""
Redis监控服务层
处理Redis监控相关的业务逻辑
"""
from typing import Optional, Dict, Any
import logging

logger = logging.getLogger(__name__)


class RedisMonitorService:
    """Redis监控服务类"""
    
    def __init__(self, namespace_data_access):
        """
        初始化Redis监控服务
        
        Args:
            namespace_data_access: 命名空间数据访问实例
        """
        self.namespace_data_access = namespace_data_access
    
    async def get_redis_monitor_data(
        self,
        namespace: str
    ) -> Dict[str, Any]:
        """
        获取Redis监控数据
        
        Args:
            namespace: 命名空间
            
        Returns:
            Redis监控数据
        """
        # TODO: 实现获取Redis监控数据逻辑
        return {
            "namespace": namespace,
            "status": "healthy",
            "metrics": {}
        }
    
    async def get_redis_config(
        self,
        namespace: str
    ) -> Dict[str, Any]:
        """
        获取Redis配置信息
        
        Args:
            namespace: 命名空间
            
        Returns:
            Redis配置信息
        """
        # TODO: 实现获取Redis配置逻辑
        return {
            "namespace": namespace,
            "config": {}
        }
    
    async def execute_redis_command(
        self,
        namespace: str,
        command: str,
        args: Optional[list] = None
    ) -> Any:
        """
        执行Redis命令
        
        Args:
            namespace: 命名空间
            command: Redis命令
            args: 命令参数
            
        Returns:
            命令执行结果
        """
        # TODO: 实现执行Redis命令逻辑
        return {
            "success": True,
            "result": None
        }
    
    async def get_slow_log(
        self,
        namespace: str,
        limit: int = 10
    ) -> Dict[str, Any]:
        """
        获取Redis慢查询日志
        
        Args:
            namespace: 命名空间
            limit: 返回记录数
            
        Returns:
            慢查询日志数据
        """
        logger.info(f"获取Redis慢查询日志 - namespace: {namespace}, limit: {limit}")
        # TODO: 实现获取Redis慢查询日志逻辑
        return {
            "namespace": namespace,
            "slow_queries": [],
            "total": 0
        }
    
    async def get_command_stats(
        self,
        namespace: str
    ) -> Dict[str, Any]:
        """
        获取Redis命令统计
        
        Args:
            namespace: 命名空间
            
        Returns:
            命令统计数据
        """
        logger.info(f"获取Redis命令统计 - namespace: {namespace}")
        # TODO: 实现获取Redis命令统计逻辑
        return {
            "namespace": namespace,
            "command_stats": {},
            "total_calls": 0
        }
    
    async def get_stream_stats(
        self,
        namespace: str,
        stream_name: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        获取Redis Stream统计
        
        Args:
            namespace: 命名空间
            stream_name: Stream名称(可选)
            
        Returns:
            Stream统计数据
        """
        logger.info(f"获取Redis Stream统计 - namespace: {namespace}, stream: {stream_name}")
        # TODO: 实现获取Redis Stream统计逻辑
        return {
            "namespace": namespace,
            "stream_name": stream_name,
            "streams": [],
            "total_messages": 0
        }


"""
命名空间管理路由
提供命名空间的增删改查和管理功能
"""
from fastapi import APIRouter, HTTPException, Query, Path
from typing import Optional, List, Dict, Any
import logging
import traceback

from jettask.schemas import NamespaceCreate, NamespaceUpdate, NamespaceResponse, NamespaceStatisticsResponse
from jettask.webui.services.settings_service import SettingsService

logger = logging.getLogger(__name__)

# 创建独立的命名空间路由，设置 /namespaces 前缀
router = APIRouter(prefix="/namespaces", tags=["namespaces"])


@router.get(
    "/",
    summary="列出所有命名空间",
    description="获取系统中所有命名空间的列表，支持分页和按状态筛选",
    response_model=List[NamespaceResponse],
    responses={
        200: {
            "description": "成功返回命名空间列表"
        },
        500: {
            "description": "服务器内部错误",
            "content": {
                "application/json": {
                    "example": {"detail": "获取命名空间列表失败: Database error"}
                }
            }
        }
    }
)
async def list_namespaces(
    page: int = Query(1, ge=1, description="页码，从 1 开始", example=1),
    page_size: int = Query(20, ge=1, le=100, description="每页数量，范围 1-100", example=20),
    is_active: Optional[bool] = Query(None, description="是否只返回激活的命名空间", example=True)
) -> List[Dict[str, Any]]:
    """
    ## 列出所有命名空间

    获取系统中所有命名空间的列表，每个命名空间包含配置信息和连接状态。

    **功能说明**:
    - 支持分页查询，避免一次返回过多数据
    - 可按激活状态筛选命名空间
    - 返回每个命名空间的完整配置信息

    **使用场景**:
    - 命名空间管理页面
    - 环境切换选择列表
    - 系统配置概览

    **示例请求**:
    ```bash
    # 获取第一页的命名空间列表
    curl -X GET "http://localhost:8001/api/v1/namespaces/?page=1&page_size=20"

    # 只获取激活的命名空间
    curl -X GET "http://localhost:8001/api/v1/namespaces/?is_active=true"
    ```

    **注意事项**:
    - 返回的 Redis 和 PostgreSQL URL 会隐藏密码信息
    - 分页参数超出范围时返回空列表
    """
    try:
        return await SettingsService.list_namespaces(page, page_size, is_active)
    except Exception as e:
        logger.error(f"获取命名空间列表失败: {e}")
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))


@router.post(
    "/",
    summary="创建新的命名空间",
    description="创建一个新的命名空间，可以选择直接配置或使用 Nacos 配置",
    response_model=NamespaceResponse,
    status_code=201,
    responses={
        201: {
            "description": "命名空间创建成功"
        },
        400: {
            "description": "请求参数错误",
            "content": {
                "application/json": {
                    "example": {"detail": "命名空间名称已存在"}
                }
            }
        },
        500: {
            "description": "服务器内部错误"
        }
    }
)
async def create_namespace(
    namespace: NamespaceCreate
) -> Dict[str, Any]:
    """
    ## 创建新的命名空间

    创建一个新的命名空间，用于隔离不同环境或项目的任务队列。

    **配置模式**:
    1. **direct 模式**: 直接提供 Redis 和 PostgreSQL 连接字符串
    2. **nacos 模式**: 通过 Nacos 配置键获取连接字符串

    **请求体参数**:
    ```json
    {
        "name": "production",
        "description": "生产环境命名空间",
        "config_mode": "direct",
        "redis_url": "redis://:password@localhost:6379/0",
        "pg_url": "postgresql://user:password@localhost:5432/jettask"
    }
    ```

    **使用场景**:
    - 创建新的环境隔离（如 dev、staging、production）
    - 多项目隔离
    - 多租户场景

    **示例请求**:
    ```bash
    # 使用直接配置模式
    curl -X POST "http://localhost:8001/api/v1/namespaces/" \\
      -H "Content-Type: application/json" \\
      -d '{
        "name": "production",
        "description": "生产环境",
        "config_mode": "direct",
        "redis_url": "redis://:password@localhost:6379/0",
        "pg_url": "postgresql://user:password@localhost:5432/jettask"
      }'

    # 使用 Nacos 配置模式
    curl -X POST "http://localhost:8001/api/v1/namespaces/" \\
      -H "Content-Type: application/json" \\
      -d '{
        "name": "staging",
        "description": "预发布环境",
        "config_mode": "nacos",
        "redis_nacos_key": "staging.redis.url",
        "pg_nacos_key": "staging.pg.url"
      }'
    ```

    **注意事项**:
    - 命名空间名称必须唯一
    - Redis 连接字符串格式: `redis://[password@]host:port/db`
    - PostgreSQL 连接字符串格式: `postgresql://user:password@host:port/database`
    - 创建后会自动验证连接是否可用
    """
    try:
        return await SettingsService.create_namespace(namespace)
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        logger.error(f"创建命名空间失败: {e}")
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/{namespace_name}",
    summary="获取命名空间详细信息",
    description="获取指定命名空间的完整配置信息和状态",
    response_model=NamespaceResponse,
    responses={
        200: {
            "description": "成功返回命名空间详情"
        },
        404: {
            "description": "命名空间不存在",
            "content": {
                "application/json": {
                    "example": {"detail": "命名空间 'production' 不存在"}
                }
            }
        },
        500: {
            "description": "服务器内部错误"
        }
    }
)
async def get_namespace(
    namespace_name: str = Path(..., description="命名空间名称", example="default")
) -> Dict[str, Any]:
    """
    ## 获取命名空间详细信息

    获取指定命名空间的完整配置信息，包括 Redis、PostgreSQL 配置和连接状态。

    **返回信息包括**:
    - 命名空间基本信息（名称、描述、启用状态）
    - Redis 连接配置
    - PostgreSQL 连接配置
    - 连接 URL
    - 配置版本号
    - 创建和更新时间

    **使用场景**:
    - 查看命名空间详细配置
    - 验证命名空间设置
    - 配置管理界面

    **示例请求**:
    ```bash
    curl -X GET "http://localhost:8001/api/v1/namespaces/default"
    ```

    **注意事项**:
    - 返回的连接字符串会隐藏敏感信息（如密码）
    - 如果命名空间不存在，返回 404 错误
    """
    try:
        return await SettingsService.get_namespace(namespace_name)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"获取命名空间失败: {e}")
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))


@router.put(
    "/{namespace_name}",
    summary="更新命名空间配置",
    description="更新指定命名空间的配置信息，支持部分更新",
    response_model=NamespaceResponse,
    responses={
        200: {
            "description": "命名空间更新成功"
        },
        400: {
            "description": "请求参数错误",
            "content": {
                "application/json": {
                    "example": {"detail": "配置参数无效"}
                }
            }
        },
        404: {
            "description": "命名空间不存在",
            "content": {
                "application/json": {
                    "example": {"detail": "命名空间 'production' 不存在"}
                }
            }
        },
        500: {
            "description": "服务器内部错误"
        }
    }
)
async def update_namespace(
    namespace_name: str = Path(..., description="命名空间名称", example="default"),
    namespace: NamespaceUpdate = ...
) -> Dict[str, Any]:
    """
    ## 更新命名空间配置

    更新指定命名空间的配置信息，所有字段都是可选的，只更新提供的字段。

    **可更新的字段**:
    - 描述信息
    - 配置模式（direct/nacos）
    - Redis 连接配置
    - PostgreSQL 连接配置
    - 启用状态

    **请求体参数**:
    ```json
    {
        "description": "更新后的描述",
        "enabled": true
    }
    ```

    **使用场景**:
    - 修改命名空间描述
    - 更新连接配置
    - 启用或禁用命名空间

    **示例请求**:
    ```bash
    # 更新描述和状态
    curl -X PUT "http://localhost:8001/api/v1/namespaces/default" \\
      -H "Content-Type: application/json" \\
      -d '{
        "description": "默认命名空间（已更新）",
        "enabled": true
      }'

    # 更新 Redis 配置
    curl -X PUT "http://localhost:8001/api/v1/namespaces/production" \\
      -H "Content-Type: application/json" \\
      -d '{
        "config_mode": "direct",
        "redis_url": "redis://:newpassword@localhost:6380/1"
      }'
    ```

    **注意事项**:
    - 更新配置后会自动验证新的连接是否可用
    - 如果有任务正在处理，建议谨慎更新
    - 只提供需要更新的字段，未提供的字段保持不变
    """
    try:
        return await SettingsService.update_namespace(namespace_name, namespace)
    except ValueError as e:
        status_code = 404 if "不存在" in str(e) else 400
        raise HTTPException(status_code=status_code, detail=str(e))
    except Exception as e:
        logger.error(f"更新命名空间失败: {e}")
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))


@router.delete(
    "/{namespace_name}",
    summary="删除命名空间",
    description="删除指定的命名空间，默认命名空间不能删除",
    responses={
        200: {
            "description": "命名空间删除成功",
            "content": {
                "application/json": {
                    "example": {"success": True, "message": "命名空间已删除"}
                }
            }
        },
        400: {
            "description": "操作不允许",
            "content": {
                "application/json": {
                    "example": {"detail": "默认命名空间不能删除"}
                }
            }
        },
        404: {
            "description": "命名空间不存在",
            "content": {
                "application/json": {
                    "example": {"detail": "命名空间 'production' 不存在"}
                }
            }
        },
        500: {
            "description": "服务器内部错误"
        }
    }
)
async def delete_namespace(
    namespace_name: str = Path(..., description="命名空间名称", example="staging")
) -> Dict[str, Any]:
    """
    ## 删除命名空间

    删除指定的命名空间及其所有相关配置。

    **删除规则**:
    - 默认命名空间（default）不能删除
    - 删除前会检查是否还有活跃的任务
    - 删除操作不可逆，请谨慎操作

    **使用场景**:
    - 清理不再使用的环境
    - 移除测试命名空间
    - 环境下线

    **示例请求**:
    ```bash
    curl -X DELETE "http://localhost:8001/api/v1/namespaces/staging"
    ```

    **注意事项**:
    - 删除命名空间会清除该命名空间下的所有配置
    - 不会删除 Redis 和 PostgreSQL 中的实际数据
    - 建议删除前先备份重要数据
    - 如果有任务正在处理，删除操作可能会失败
    """
    try:
        return await SettingsService.delete_namespace(namespace_name)
    except ValueError as e:
        status_code = 400 if "默认命名空间" in str(e) else 404
        raise HTTPException(status_code=status_code, detail=str(e))
    except Exception as e:
        logger.error(f"删除命名空间失败: {e}")
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))



@router.get(
    "/{namespace_name}/statistics",
    summary="获取命名空间统计信息",
    description="获取指定命名空间的统计数据，包括队列数、任务数、Worker 数等",
    response_model=NamespaceStatisticsResponse,
    responses={
        200: {
            "description": "成功返回统计信息"
        },
        404: {
            "description": "命名空间不存在",
            "content": {
                "application/json": {
                    "example": {"detail": "命名空间 'production' 不存在"}
                }
            }
        },
        500: {
            "description": "服务器内部错误"
        }
    }
)
async def get_namespace_statistics(
    namespace_name: str = Path(..., description="命名空间名称", example="default")
) -> Dict[str, Any]:
    """
    ## 获取命名空间统计信息

    获取指定命名空间的实时统计数据，用于监控和分析。

    **统计指标包括**:
    - 队列总数
    - 任务总数
    - 活跃 Worker 数
    - Redis 内存使用情况
    - 数据库连接数

    **使用场景**:
    - 命名空间监控面板
    - 资源使用分析
    - 容量规划
    - 性能优化

    **示例请求**:
    ```bash
    curl -X GET "http://localhost:8001/api/v1/namespaces/default/statistics"
    ```

    **返回示例**:
    ```json
    {
        "success": true,
        "data": {
            "total_queues": 5,
            "total_tasks": 1250,
            "active_workers": 12,
            "redis_memory_usage": 10485760,
            "db_connections": 5
        },
        "namespace": "default"
    }
    ```

    **注意事项**:
    - 统计数据为实时查询，可能会有轻微延迟
    - Redis 内存使用单位为字节（bytes）
    - 如果命名空间未启用，某些统计数据可能为 0
    """
    try:
        return await SettingsService.get_namespace_statistics(namespace_name)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"获取命名空间统计信息失败: {e}")
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e))

__all__ = ['router']


"""
定时任务调度模块

文件说明：
- models.py: 数据模型（ScheduledTask, TaskExecutionHistory）
- schedule.py: Schedule 定义类（用于定义定时任务）
- scheduler.py: 核心调度器（TaskScheduler）
- loader.py: 任务加载器（TaskLoader）
- database.py: 数据库操作（ScheduledTaskManager）
- manager.py: 统一调度器管理器（UnifiedSchedulerManager，用于CLI）
- sql/: SQL文件目录
  - schema.sql: 数据库表结构
  - migrations/: 数据库迁移脚本
"""

from .models import ScheduledTask, TaskExecutionHistory
from .scheduler import TaskScheduler
from .loader import TaskLoader
from .database import ScheduledTaskManager
from .schedule import Schedule

__all__ = [
    'ScheduledTask',
    'TaskExecutionHistory',
    'TaskScheduler',
    'TaskLoader',
    'ScheduledTaskManager',
    'Schedule'
]


"""
命名空间依赖注入工具
提供统一的命名空间参数处理和数据库连接管理
"""
import logging
from typing import Optional
from fastapi import Depends, HTTPException, Path, Request
from jettask.core.namespace import NamespaceConnection, NamespaceDataAccessManager

logger = logging.getLogger(__name__)


class NamespaceContext:
    """
    命名空间上下文对象
    封装命名空间的所有信息和数据库连接
    """

    def __init__(
        self,
        namespace_name: str,
        connection: NamespaceConnection,
        manager: NamespaceDataAccessManager
    ):
        self.namespace_name = namespace_name
        self.connection = connection
        self.manager = manager

    async def get_redis_client(self, decode: bool = True):
        """获取 Redis 客户端"""
        return await self.connection.get_redis_client(decode=decode)

    async def get_pg_session(self):
        """获取 PostgreSQL 会话"""
        return await self.connection.get_pg_session()

    @property
    def redis_prefix(self) -> str:
        """获取 Redis 键前缀"""
        return self.connection.redis_prefix

    @property
    def redis_config(self) -> dict:
        """获取 Redis 配置"""
        return self.connection.redis_config

    @property
    def pg_config(self) -> dict:
        """获取 PostgreSQL 配置"""
        return self.connection.pg_config


def get_namespace_manager(request: Request) -> NamespaceDataAccessManager:
    """
    获取命名空间数据访问管理器
    从 app.state 中获取全局实例
    """
    if not hasattr(request.app.state, 'namespace_data_access'):
        raise HTTPException(
            status_code=500,
            detail="Namespace data access not initialized"
        )

    # namespace_data_access 是 NamespaceJetTaskDataAccess 实例
    # 它包含一个 manager 属性
    return request.app.state.namespace_data_access.manager


async def get_namespace_context(
    namespace: str = Path(..., description="命名空间名称", example="default"),
    manager: NamespaceDataAccessManager = Depends(get_namespace_manager)
) -> NamespaceContext:
    """
    获取命名空间上下文（依赖注入函数）

    这个函数会：
    1. 从路径参数中提取 namespace 名称
    2. 从任务中心API或Nacos获取命名空间配置
    3. 建立数据库连接（Redis + PostgreSQL）
    4. 返回封装好的命名空间上下文对象

    使用示例：
    ```python
    @router.get("/{namespace}/queues")
    async def get_queues(
        ns: NamespaceContext = Depends(get_namespace_context)
    ):
        # 直接使用 ns 对象访问数据库
        redis_client = await ns.get_redis_client()
        # ... 执行操作
        return {"namespace": ns.namespace_name}
    ```

    Args:
        namespace: 从路径参数中提取的命名空间名称
        manager: 命名空间数据访问管理器（自动注入）

    Returns:
        NamespaceContext: 命名空间上下文对象

    Raises:
        HTTPException: 当命名空间不存在或配置错误时
    """
    try:
        # 获取命名空间的数据库连接
        connection = await manager.get_connection(namespace)

        # 创建并返回上下文对象
        return NamespaceContext(
            namespace_name=namespace,
            connection=connection,
            manager=manager
        )

    except ValueError as e:
        logger.error(f"命名空间 '{namespace}' 配置错误: {e}")
        raise HTTPException(
            status_code=404,
            detail=f"命名空间 '{namespace}' 不存在或配置错误"
        )
    except Exception as e:
        logger.error(f"获取命名空间 '{namespace}' 上下文失败: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"获取命名空间连接失败: {str(e)}"
        )


# 为了向后兼容，提供一个简化的依赖函数
async def get_namespace_connection(
    namespace: str = Path(..., description="命名空间名称", example="default"),
    manager: NamespaceDataAccessManager = Depends(get_namespace_manager)
) -> NamespaceConnection:
    """
    获取命名空间数据库连接（简化版本）

    直接返回 NamespaceConnection 对象，适用于简单场景

    Args:
        namespace: 命名空间名称
        manager: 命名空间数据访问管理器

    Returns:
        NamespaceConnection: 数据库连接对象
    """
    try:
        return await manager.get_connection(namespace)
    except ValueError as e:
        logger.error(f"命名空间 '{namespace}' 配置错误: {e}")
        raise HTTPException(
            status_code=404,
            detail=f"命名空间 '{namespace}' 不存在或配置错误"
        )
    except Exception as e:
        logger.error(f"获取命名空间 '{namespace}' 连接失败: {e}", exc_info=True)
        raise HTTPException(
            status_code=500,
            detail=f"获取命名空间连接失败: {str(e)}"
        )


__all__ = [
    'NamespaceContext',
    'get_namespace_context',
    'get_namespace_connection',
    'get_namespace_manager'
]



"""
数据库模块

提供统一的数据库模型定义、操作接口和连接管理
"""

from .base import Base, get_engine, get_session, init_db
from .connector import (
    # 连接池函数
    get_sync_redis_pool,
    get_async_redis_pool,
    get_async_redis_pool_for_pubsub,
    get_pg_engine_and_factory,
    get_asyncpg_pool,
    # 客户端实例函数
    get_sync_redis_client,
    get_async_redis_client,
    get_dual_mode_async_redis_client,
    # 缓存清理
    clear_all_cache,
    # 配置解析
    DBConfig,
    # 连接器类
    SyncRedisConnector,
    RedisConnector,
    PostgreSQLConnector,
    ConnectionManager,
    # 便捷函数
    create_redis_client,
    create_pg_session,
)

__all__ = [
    # SQLAlchemy 基础
    'Base',
    'get_engine',
    'get_session',
    'init_db',

    # 连接池函数
    'get_sync_redis_pool',
    'get_async_redis_pool',
    'get_async_redis_pool_for_pubsub',
    'get_pg_engine_and_factory',
    'get_asyncpg_pool',

    # 客户端实例函数
    'get_sync_redis_client',
    'get_async_redis_client',
    'get_dual_mode_async_redis_client',

    # 缓存清理
    'clear_all_cache',

    # 配置解析
    'DBConfig',

    # 连接器类
    'SyncRedisConnector',
    'RedisConnector',
    'PostgreSQLConnector',
    'ConnectionManager',

    # 便捷函数
    'create_redis_client',
    'create_pg_session',
]
